#!/usr/bin/env python
# -*- coding: utf-8 -*-

import subprocess
import sys


def test():
    cmd = "./rna_rosetta_run.py -r -g -n 10 -c 1 --sandbox test_data test_data/tetraloop.fa"
    # - i - e
    o = subprocess.Popen(cmd, shell=True, stdout=sys.stdout.flush(), stderr=subprocess.PIPE)
    stderr = o.stderr.read().strip()
    assert stderr == ''

def test_cluster():
    cmd = "./rna_rosetta_cluster.py test_data/selected.out 10"
    o = subprocess.Popen(cmd, shell=True, stdout=sys.stdout.flush(), stderr=subprocess.PIPE)
    stderr = o.stderr.read().strip()
    assert stderr == ''



if __name__ == '__main__':
    test()
    test_cluster()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Secondary structure analysis"""

import os
import tempfile
import shutil
import subprocess

from rna_tools.rna_tools_config import VARNA_JAR_NAME, VARNA_PATH


class ExceptionOpenPairsProblem(Exception):
    pass


def draw_ss(title, seq, ss, img_out, resolution=4, verbose=False):
    """Draw Secondary Structure using VARNA (you need correct configuration for this).

    If everything is OK, return None, if an error (=exception) return stderr.

    Usage::

       >>> seq = 'GGAAACC'
       >>> ss =  '((...))'
       >>> img_out = 'output/demo.png'
       >>> draw_ss('rna', seq, ss, img_out)
       >>> print('Made %s' % img_out)
       Made output/demo.png

    .. image:: ../../rna_tools/output/demo.png
       :scale: 25 %

    Can be used with http://geekbook.readthedocs.io/en/latest/rna.html"""
    curr = os.getcwd()
    os.chdir(VARNA_PATH)  # VARNAv3-93-src')
    if verbose:
        print(VARNA_PATH)
    t = tempfile.NamedTemporaryFile(delete=False)
    t.name += '.png'

    cmd = 'java -cp ' + VARNA_JAR_NAME + ' fr.orsay.lri.varna.applications.VARNAcmd -sequenceDBN ' + seq + \
        " -structureDBN '" + ss + "' -o " + t.name + " -title '" + \
        title + "' -resolution '" + str(resolution) + "'"
    if verbose:
        print(cmd)
    p = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    p.wait()
    out = p.stderr.read().decode().strip()
    os.chdir(curr)
    if out.find('Exception') > -1:
        return out
    else:
        if verbose:
            print(t.name)
        shutil.move(t.name, img_out)


def parse_vienna_to_pairs(ss, remove_gaps_in_ss=False):
    """Parse Vienna (dot-bracket notation) to get pairs.

    Args:

       ss (str): secondary stucture in Vienna (dot-bracket notation) notation
       remove_gaps_in_ss (bool): remove - from ss or not, design for DCA (tpp case
                                 ``ss = "(((((((((.((((.(((.....))))))......------)....."``
                                 works with pk of the first level, ``[[]]``

    Returns:

        list of two lists: (pairs, pairs_pk)

    Examples::

        >>> parse_vienna_to_pairs('((..))')
        ([[1, 6], [2, 5]], [])

        >>> parse_vienna_to_pairs('(([[))]]')
        ([[1, 6], [2, 5]], [[3, 8], [4, 7]])

        >>> parse_vienna_to_pairs('((--))')
        ([[1, 6], [2, 5]], [])

        >>> parse_vienna_to_pairs('((--))', remove_gaps_in_ss=True)
        ([[1, 4], [2, 3]], [])

        >>> parse_vienna_to_pairs('((((......')
        Traceback (most recent call last):
          File "/usr/lib/python2.7/doctest.py", line 1315, in __run
            compileflags, 1) in test.globs
          File "<doctest __main__.parse_vienna_to_pairs[4]>", line 1, in <module>
            parse_vienna_to_pairs('((((......')
          File "./SecondaryStructure.py", line 106, in parse_vienna_to_pairs
            raise ExceptionOpenPairsProblem('Too many open pairs (()) in structure')
        ExceptionOpenPairsProblem: Too many open pairs (()) in structure

    """
    if remove_gaps_in_ss:
        ss = ss.replace('-', '')
    stack = []
    pairs = []
    pairs_pk = []
    stack_pk = []
    for c, s in enumerate(ss):
        if s == '(':
            stack.append(c + 1)
        if s == ')':
            pairs.append([stack.pop(), c + 1])
        if s == '[':
            stack_pk.append(c + 1)
        if s == ']':
            pairs_pk.append([stack_pk.pop(), c + 1])

    if stack:
        raise ExceptionOpenPairsProblem('Too many open pairs (()) in structure')
    if stack_pk:
        raise ExceptionOpenPairsProblem('Too many open pairs [[]] in structure')

    pairs.sort()
    pairs_pk.sort()
    return(pairs, pairs_pk)


# main
if __name__ == '__main__':
    import doctest
    doctest.testmod()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Keep only for reference. TO BE REMOVE!!!!

SLOPE A/23/MB C/45/MB 0 6 2.0
SLOPE A/23/MB C/45/MB 0 7 -2.0

A16-A82 < 100 1

"""
from __future__ import print_function
import sys


import argparse


def get_parser():
    parser = argparse.ArgumentParser(
        description=__doc__, formatter_class=argparse.RawDescriptionHelpFormatter)

    parser.add_argument("--offset", help="can be -10", default=0, type=int)
    parser.add_argument("--weight", help="", default=1, type=float)
    parser.add_argument("-v", "--verbose",
                        action="store_true", help="be verbose")
    parser.add_argument('file')
    return parser


if __name__ == '__main__':
    parser = get_parser()
    args = parser.parse_args()

    # 13 -> 1
    offset = args.offset
    fn = args.file
    weight = args.weight
    #
    for l in open(fn):
        if l.startswith('#'):
            continue

        # d:A58-A68 < 100 1
        l = l.replace('d:', '')
        l = l.replace('A', '')
        pair, rest = l.split('<')
        a, b = pair.split('-')

        a = int(a.strip())
        a += offset
        b = int(b.strip())
        b += offset

        a = str(a)
        b = str(b)

        print('SLOPE A/' + a + '/MB A/' + b + '/MB 0 6 ' + str(weight))
        print('SLOPE A/' + a + '/MB A/' + b + '/MB 0 7 -' + str(weight))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""rna_simrna_extra.py - extract full atom structures from a SimRNA trajectory file

Options:

  SIMRNA_DATA_PATH has to be properly defined in ``rpt_config_local``.

"""
from rna_tools.tools.simrna_trajectory.simrna_trajectory import SimRNATrajectory
from rna_tools.rna_tools_config import SIMRNA_DATA_PATH

import argparse
import os

import logging
logger = logging.getLogger()
logger.setLevel(logging.DEBUG)
# create file handler which logs even debug messages
fh = logging.FileHandler('rna_simrna_extract.log')
fh.setLevel(logging.INFO)
formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
fh.setFormatter(formatter)
logger.addHandler(fh)


def get_parser():
    """Get parser of arguments."""
    parser = argparse.ArgumentParser(
        description=__doc__, formatter_class=argparse.RawDescriptionHelpFormatter)
    parser.add_argument(
        '-t', '--template',
        help="template PDB file used for reconstruction to full atom models", required=True)
    parser.add_argument('-f', '--trafl', help="SimRNA trafl file", required=True)
    parser.add_argument('-c', '--cleanup', action='store_true',
                        help="Keep only *_AA.pdb files, move *.ss_detected and *.pdb"
                        "to _<traj name folder>")
    parser.add_argument('-n', '--number_of_structures', help="", default=100)
    return parser


def get_data():
    """Get a link to SimRNA data folder in cwd."""
    cmd = 'ln -s %s %s' % (SIMRNA_DATA_PATH, os.getcwd())
    print('getting SimRNA data folder in cwd ...', cmd)
    logger.info(cmd)
    os.system(cmd)


def extract(template, trafl, number_of_structures=''):
    """Run SimRNA_trafl2pdb to extract all full atom structures in the trajectory."""
    cmd = 'SimRNA_trafl2pdbs %s %s AA :%s' % (template, trafl, number_of_structures)
    # cmd = 'SimRNA_trafl2pdbs %s %s AA : ' % (template, trafl)  # like extract all :
    print('rna_simrna_extract::' + cmd)
    os.system(cmd)


def cleanup(trafl):
    """Create _<trafl> with all CG structures and ss_detected and in current directory
    keep only full atom structures.

    e.g ::

            _1db3ee42-d2b1-4c6e-b81c-92f972e03310_ALL_100low

            [mm] sim_a99 ls _1* | head
            1db3ee42-d2b1-4c6e-b81c-92f972e03310_ALL_100low-000001.pdb
            1db3ee42-d2b1-4c6e-b81c-92f972e03310_ALL_100low-000001.ss_detected
            1db3ee42-d2b1-4c6e-b81c-92f972e03310_ALL_100low-000002.pdb
            1db3ee42-d2b1-4c6e-b81c-92f972e03310_ALL_100low-000002.ss_detected

    Now, in the folder there will be only AA.pdb
    find can be used to remove files, if rm can't handle, e.g. with 10k files.
    """
    traflfn = trafl.replace('.trafl', '')
    try:
        os.mkdir('_%s' % traflfn)
    except OSError:
        pass

    # move AA to a folder, .e.g _ade_pk-35b2a2c1_ALL_top500
    cmd = 'mv -v %s*AA.pdb _%s' % (traflfn, traflfn)
    print(cmd)
    os.system(cmd)

    # mv vs trash them
    if True:
        # os.system('mv %s*.pdb _%s' % (traflfn, traflfn))
        os.system('rm %s*.pdb' % traflfn)
        # os.system('find . -name "*%s*.pdb" -print0 | xargs -0 rm ' % traflfn)
        # os.system('mv %s*.ss_detected _%s' % (traflfn, traflfn))
        os.system('rm %s*.ss_detected' % traflfn)
        # os.system('find . -name "*%s*ss_detected" -print0 | xargs -0 rm ' % traflfn)
        pass

if __name__ == '__main__':
    parser = get_parser()
    args = parser.parse_args()

    get_data()
    extract(args.template, args.trafl, args.number_of_structures)
    if args.cleanup:
        cleanup(os.path.basename(args.trafl))



#!/usr/bin/env python
"""renum_pdb_to_aln.py - renumber a pdb file based on the alignment.

author: A. Zyla under supervision of mmagnus

.. warning:: works only for single chain! and requires Biopython (tested with v1.68)
"""

import logging
import argparse
from Bio.SeqRecord import SeqRecord
from Bio import SeqIO
from Bio.PDB import PDBParser
from Bio.PDB import PDBIO
from Bio.PDB.Atom import PDBConstructionWarning
import warnings
warnings.simplefilter('ignore', PDBConstructionWarning)

# logger
logger = logging.getLogger()
handler = logging.StreamHandler()
logger.addHandler(handler)


def get_seq(alignfn, seqid):
    """Get seq from an alignment with gaps.

    Args:

       alignfn (str): a path to an alignment
       seqid   (str): seq id in an alignment

    Usage::

        >>> get_seq('test_data/ALN_OBJ1_OBJ2.fa', 'obj1')
        SeqRecord(seq=SeqRecord(seq=Seq('GUUCAG-------------------UGAC-', SingleLetterAlphabet()), id='obj1', name='obj1', description='obj1', dbxrefs=[]), id='<unknown id>', name='<unknown name>', description='<unknown description>', dbxrefs=[])

    Returns:
         SeqRecord
    """
    # alignment = AlignIO.read(alignfn, 'fasta')
    alignment = SeqIO.index(alignfn, 'fasta')
    # print SeqRecord(alignment[seqid])
    sequence = SeqRecord(alignment[seqid])
    return sequence


def open_pdb(pdbfn):
    """Open pdb with Biopython.

    Args:
       pdbfn (str): a path to a pdb structure

    Returns:
       PDB Biopython object: with a pdb structure

    """
    parser = PDBParser()
    return parser.get_structure('struc', pdbfn)


def renumber(seq_with_gaps, struc, residue_index_start):
    """Renumber a pdb file.

    Args:
        seq_with_gaps (str): a target sequence extracted from the alignment
        struc         (pdb): a structure
        residue_index_start (int): starting number

    Returns:
        BioPython Structure object

    """
    new_numbering = []
    for nt in seq_with_gaps:
        if nt != '-':
            nt_num_a = [residue_index_start, nt]
            new_numbering.append(residue_index_start)
            logger.info(nt_num_a)
        residue_index_start = residue_index_start + 1

    logger.info(new_numbering)

    # works only for single chain
    for struc in pdb:
        for chain in struc:
            for residue, resi in zip(chain, new_numbering):
                residue.id = (residue.id[0], resi, residue.id[2])
    return struc


def write_struc(struc, outfn):
    """Write renumbered pdb with Biopython.

    Args:
        struc (pdb): a renumbered structure
        outfn (str): a path to a new, renumbered pdb file

    Returns:
        none: writes to a file

    """
    io = PDBIO()
    io.set_structure(struc)
    io.save(outfn)
    logger.info('Structure written to %s' % outfn)


def get_parser():
    parser = argparse.ArgumentParser(description=__doc__,
                                     formatter_class=argparse.RawDescriptionHelpFormatter)
    parser.add_argument("-v", "--verbose", help="increase output verbosity",
                        action="store_true")

    parser.add_argument("--residue_index_start",
                        help="renumber starting number (default: 1)",
                        default=1, type=int)
    parser.add_argument("--outfn", help="output pdb file (default: pdbfn .pdb -> _out.pdb)")
    parser.add_argument("seqid", help="seq id in the alignemnt")
    parser.add_argument("alignfn", help="alignemnt in the Fasta format")
    parser.add_argument("pdbfn", help="pdb file")
    return parser


# main
if __name__ == '__main__':
    args = get_parser().parse_args()

    if args.verbose:
        logger.setLevel(logging.INFO)

    if not args.outfn:
        args.outfn = args.pdbfn.replace('.pdb', '_out.pdb')

    seq_with_gaps = get_seq(args.alignfn, args.seqid)
    pdb = open_pdb(args.pdbfn)
    struc = renumber(seq_with_gaps, pdb, args.residue_index_start)
    write_struc(struc, args.outfn)



#!/usr/bin/env python
r"""rna_plot_density.py - generate a density plot

Don't open Excel, Jupyter. Simple plot a density of one column and save it to a file.

Example::

    # file
                                                      fn  rmsd_all
    0  19_Bujnicki_Human_4_rpr_n0-000001.pdb-000001_A...     14.73
    1  19_Bujnicki_Human_4_rpr_n0-000001.pdb.trafl_19...      0.46
    2  19_Bujnicki_Human_4_rpr_n0-000001.pdb.trafl_19...     14.73
    3  19_Bujnicki_Human_4_rpr_n0-000001.pdb_thrs0.50...      0.73
    4  19_Bujnicki_Human_4_rpr_n0-000001.pdb_thrs0.50...      0.83

    $ rna_plot_hist.py rmsds.csv --column rmsd_all

.. image:: ../../rna_tools/tools/plotting/test_data/rmsds_dens.png
"""
from __future__ import print_function
import pandas as pd
import matplotlib.pyplot as plt
import argparse
import sys

plt.style.use('ggplot')
plt.rc('figure', figsize=(10, 6))


def get_parser():
    """Get parser."""
    parser = argparse.ArgumentParser(description=__doc__,
                                     formatter_class=argparse.RawDescriptionHelpFormatter)
    parser.add_argument('file', help="rmsd.txt")
    parser.add_argument('--column', help="column of file to plot")
    parser.add_argument('--sep', help="separator, be default \t", default=",")
    parser.add_argument('-o', '--output')
    return parser


if __name__ == '__main__':
    parser = get_parser()
    args = parser.parse_args()

    fn = args.file
    df = pd.read_csv(args.file, sep=args.sep)
    print(df.head())

    if not args.column:
        print('Set column')
        sys.exit(0)

    ax = df[args.column].plot(kind='density')

    if not args.output:
        outfn = args.file.replace('.txt', '').replace('.csv', '') + '_dens.png'
        print('Save plot %s' % outfn)
        plt.savefig(outfn)



#!/usr/bin/python

from Bio.PDB import NeighborSearch, PDBParser, Selection, Atom
from numpy import array

def check_clash(str_name, v=True):
        """check_clash, fract of clashes!

        if zero contacts then error -> fix ->

        Problem, contacts, str_name: 311 505 na-prot_13536.pdb
        Sterical clashes  0.615841584158

        c is counter
        """
        print(str_name)
        structure = open(str_name)
        #model = structure[0]
        atoms_A = []
        atoms_B = []
        for line in structure.readlines():
            if line[:4] == "ATOM":
                #print line
                at_nam = line[12:16].strip()
                coor = [float(line[30:38]),float(line[38:46]), float(line[46:54])]	
                at = Atom.Atom(at_nam,coor,0.0,1.0,' ',at_nam,1,at_nam[0])
                if line[21] == "A":
                    atoms_A.append(at)
                elif line[21] == "B":
                    atoms_B.append(at)
                else: pass
        #atoms_B = Selection.unfold_entities(structure[0]['B'], 'A')
        #print len(atoms_A), len(atoms_B)
        if len(atoms_A) > len(atoms_B):
            less = atoms_B
            more = atoms_A
        else: 
            less = atoms_A
            more = atoms_B
        problem = 0
        contacts = 0 
        ns=NeighborSearch(more)
        for at in less:
             neighbors=ns.search(array(at.get_coord()),2.0,'A')
             if neighbors != []:
                 problem +=1
                 contacts +=1
             else:
                 neighbors1=ns.search(array(at.get_coord()),4.0,'A')
                 if neighbors1 != []:
                     contacts +=1
        if v:
                print('problem:', float(problem))
                print('contacts:', float(contacts))
        try:
            fract = float(problem)/float(contacts)
        except ZeroDivisionError:
            fract = problem # or skip this structure
            print('ZeroDivison -- skip:', problem, contacts, str_name)
            return fract

        #print 'Contacts, str_name:', problem, contacts, str_name, "Sterical clashes ", fract
        return fract
    
if __name__ == '__main__':
    print(check_clash('test_data/no_clash.pdb'))
    print(check_clash('test_data/super_clash.pdb'))
    print(check_clash('test_data/prot-na_2392.pdb'))



#!/usr/bin/env python

"""
Example::

    ./rna_helix_vis.py -p test_data/rp14_farna_eloop.out.1.pdb -s test_data/rp14.ss
    (((((.(((((....)))))..[.....((((((..........))))))....)))))].

    /// copy below to pymol ///
    from pymol.cgo import *
    from pymol import cmd
    color red, resi 1+2+3+4+5+55+56+57+58+59
    color red, resi 7+8+9+10+11+16+17+18+19+20
    color red, resi 29+30+31+32+33+34+45+46+47+48+49+50
    obj = [ CYLINDER, -4.89364,-21.9653,-24.0907,10.3337,-22.0812,-28.8867, 9, 1, 0, 0, 1,0,0, CYLINDER, 9.52677,-27.707,-35.113,4.12388,-23.5705,-48.3105, 9, 1, 0, 0, 1,0,0, CYLINDER, 3.48367,-49.4752,-18.6601,-12.6687,-57.1961,-17.8483, 9, 1, 0, 0, 1,0,0,]
    cmd.load_cgo(obj,'cgo01')
    /// copy above to pymol ///

.. image:: ../../rna_tools/tools/rna_helix_vis/doc/rna_helix_vis.png

.. warning:: works only for chain A @todo

required: forgi (https://github.com/pkerpedjiev/forgi)

`pip install forgi`
"""
import argparse
import sys
import six

from rna_tools.tools.rna_bp.base_pair import *

if six.PY2:
    import forgi.graph.bulge_graph as fgb
    bg = fgb.BulgeGraph()
else:
    print('Run under Python 2')


class Helix:
    def __init__(self, id, fn, chain, start, end):
        self.id = id
        self.start = start  # (1,2)
        self.end = end
        self.fn = fn
        self.chain = chain
        self.load_structure()

    def __repr__(self):
        return 'Helix ' + str(self.id) + ' ' + str(self.start) + '-' + str(self.end)

    def get_ids(self):
        t = ''
        for x in range(self.start[0], self.start[1] + 1):
            t += str(x) + '+'
        for x in range(self.end[0], self.end[1] + 1):
            t += str(x) + '+'
        return 'color red, resi ' + t[:-1]

    def load_structure(self):
        parser_pdb = PDB.PDBParser()
        struct = parser_pdb.get_structure('', self.fn)  # )
        model = struct[0]
        self.chain = model[self.chain]
        self.struct = struct

    def get_helix_coord(self):
        chain = self.chain
        struct = self.struct
        a = chain[self.start[0]]
        b = chain[self.end[1]]

        bp = BasePair(a, b, struct)

        a = chain[self.start[1]]
        b = chain[self.end[0]]

        bp2 = BasePair(a, b, struct)

        # print "cmd.load_cgo( [9.0, " + ','.join([str(x) for x in bp.calc_coord()]) + ',' +  ','.join([str(x) for x in bp2.calc_coord()]) + ", 5, 1, 0, 0, 1,0,0], 'cylinderx' )"
        return ','.join([str(x) for x in bp.calc_coord()]) + ',' + ','.join([str(x) for x in bp2.calc_coord()]) + ", 9, 1, 0, 0, 1,0,0"


def show_cyliders(ss):

    bg = fgb.BulgeGraph()
    bg.from_dotbracket(ss)

    helices = []

    print("""from pymol.cgo import *
from pymol import cmd""")

    ht = 'obj = [ CYLINDER, '
    for i in bg.to_bg_string().split('\n'):
        # print i
        if i.startswith('define s'):
            x = i.replace('define s', '')
            # print x
            x = [int(x) for x in x.split()]
            h = Helix(x[0], fn, chain, (x[1], x[2]), (x[3], x[4]))
            helices.append(h)
            print((h.get_ids()))
            ht += h.get_helix_coord() + ', CYLINDER, '

    print((ht[:-11] + ']'))
    print("cmd.load_cgo(obj,'cgo01') ")


def get_parser():
    parser = argparse.ArgumentParser()  # usage="%prog [<options>] <pdb files (test_data/*)>")
    parser.add_argument('-p', "--pdb_fn",
                        dest="pdb_fn",
                        default='',
                        help="pdb file")

    parser.add_argument('-s', "--ss_fn",
                        dest='ss',
                        default='',
                        help="secondary structure file, only with ss (one line, one chain)")
    return parser


if __name__ == '__main__':
    parser = get_parser()
    args = parser.parse_args()

    if len(sys.argv) == 1:
        parser.print_help()
        sys.exit(1)

    chain = 'A'
    fn = args.pdb_fn
    ss = open(args.ss).read().strip()

    print(ss)
    print('/// copy below to pymol ///')
    show_cyliders(ss)
    print('/// copy above to pymol ///')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""rna_calc_rmsd_all_vs_all.py - calculate RMSDs all vs all and save it to a matrix

Examples::

    rna_calc_rmsd_all_vs_all.py -i test_data -o test_output/rmsd_calc_dir.tsv
     # of models: 4
    ... 1 test_data/struc1.pdb
    ... 2 test_data/struc2.pdb
    ... 3 test_data/struc3.pdb
    ... 4 test_data/struc4.pdb

The program is using (https://github.com/charnley/rmsd)
"""
from __future__ import print_function

from rna_tools.tools.rna_calc_rmsd.lib.rmsd.calculate_rmsd import rmsd, get_coordinates, centroid, kabsch_rmsd

import argparse
import glob
import re
import os


def get_rna_models_from_dir(directory):
    models = []
    if not os.path.exists(directory):
        raise Exception('Dir does not exist! ', directory)
    files = glob.glob(directory + "/*.pdb")
    files_sorted = sort_nicely(files)
    for f in files_sorted:
        models.append(f)
    return models


def sort_nicely(l):
    """Sort the given list in the way that humans expect.
    http://blog.codinghorror.com/sorting-for-humans-natural-sort-order/
    """
    def convert(text): return int(text) if text.isdigit() else text

    def alphanum_key(key): return [convert(c) for c in re.split('([0-9]+)', key)]
    l.sort(key=alphanum_key)
    return l


def calc_rmsd(a, b):
    """Calc rmsd."""
    atomsP, P = get_coordinates(a, None, None, 'pdb', True)
    atomsQ, Q = get_coordinates(b, None, None, 'pdb', True)

    # Calculate 'dumb' RMSD
    # gnormal_rmsd = rmsd(P, Q)

    # Create the centroid of P and Q which is the geometric center of a
    # N-dimensional region and translate P and Q onto that center.
    # http://en.wikipedia.org/wiki/Centroid
    Pc = centroid(P)
    Qc = centroid(Q)
    P -= Pc
    Q -= Qc

    # if False:
    #    V = rotate(P, Q)
    #    V += Qc
    #    write_coordinates(atomsP, V)
    #    quit()

    return kabsch_rmsd(P, Q)


def get_parser():
    parser = argparse.ArgumentParser(description=__doc__,
                                     formatter_class=argparse.RawTextHelpFormatter)
    parser.add_argument('-i', "--input-dir",
                        default='',
                        help="input folder with structures")

    parser.add_argument('-o', "--matrix-fn",
                        default='matrix.txt',
                        help="ouput, matrix")

    # parser.add_argument("-s", "--save",
    #                    action="store_true", help="")

    return parser


if __name__ == '__main__':
    parser = get_parser()
    args = parser.parse_args()

    input_dir = args.input_dir
    matrix_fn = args.matrix_fn

    models = get_rna_models_from_dir(input_dir)

    print(' # of models:', len(models))

    f = open(matrix_fn, 'w')
    t = '# '
    for r1 in models:
        # print r1,
        t += str(r1) + ' '
    # print
    t += '\n'

    c = 1
    for r1 in models:
        for r2 in models:
            rmsd_curr = calc_rmsd(r1, r2)
            t += str(round(rmsd_curr, 3)) + ' '
        print('...', c, r1)
        c += 1
        t += '\n'

    f.write(t)
    f.close()

    print(t.strip())  # matrix

    if True:
        print('matrix was created! ', matrix_fn)
    else:
        print('matrix NOT was created!')



#!/usr/bin/env python

"""Process an alignment in the Stockholm format to get sequences and secondary structures:

Example::

    $ rna_align_get_ss_from_stk.py aligns/gmp_RF01786.stockholm_p75_pk.sto
    AAOX01000007.1/31274-31356
    AAGAAUAUAGAACACUGUGAUGAGCGGUUUUUAUUUGCACUUUAAACCGCUUGGAGUGACUAGUGCAGCCGGCCAAUGAUCUA
    .(((.(((.(..(((......((((((((...............))))))))...)))...............).))).))).
    CP000724.1/3560727-3560809
    AAAAAUGUAGAGCAAAUGAACUGCAGGUAUACAUGGACGCCUUAAACUGCAGGGAUGUAGUGGCGUAACCGACUAACAAUAUU
    ((.(.(((((.(((......((((((.(......[[...[....).))))))...)))...]...]..]...))).)).).))
    AACY023761929.1/1009-1091
    AUAAUUUGGUGGGCGUUGAUGUGCCCUUUGUAUCUGGUCGCUUGAGGGGUACGGAGCCAAUAGCGAAACCGCCGCCGUCAUAG
    .((...((((((((......((((((((.......[.[......))))))))...)))......]...]...)))))...)).

"""
from rna_tools.tools.rna_alignment.rna_alignment import clean_seq_and_ss, RNAalignment
import argparse

def get_parser():
    parser = argparse.ArgumentParser(description=__doc__, formatter_class=argparse.RawDescriptionHelpFormatter)
    parser.add_argument('file', help="subsection of an alignment")
    return parser

if __name__ == '__main__':
    args = get_parser().parse_args()
    a = RNAalignment(args.file)
    for s in a:
        s.remove_gaps()
        print(s)
        print(s.seq)
        print(s.ss)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Module for rebuilding all atom representation from reduced,
3 atom representation.
"""
import os
DIRNAME = os.path.dirname(__file__)

from subprocess import Popen
from shutil import copyfile, rmtree
from tempfile import mkdtemp
from .SingleLineUtils import get_res_num
from .PDBFile import PDBFile

REBUILDRNA_PATH=os.path.join(DIRNAME, 'RebuildRNA', 'trunk')

def check_3_atom(pdb_string):
    '''Check what number of atoms per residue in PDB

    Arguments:
      * pdb_string = string with PDB structure to check

    Output:
      * True if there are 3 atoms per residue, False otherwise
    '''
    pdb_lines = pdb_string.split('\n')
    atom_lines = [l for l in pdb_lines if l.startswith('ATOM')]
    res_nums = [get_res_num(l) for l in atom_lines]
    uniq_nums = list(set(res_nums))
    counted = [res_nums.count(i) for i in uniq_nums]
    return not [i for i in counted if i != 3]


def rebuild_full_atom(input_pdb, output_pdb):
    """Make all atom PDB file from 3 atom PDB file

    Arguments:
      * input_pdb = path to input PDB
      * output_pdb = path to output PDB
    """
    try:
        old_pwd = os.getcwd()
        print(old_pwd)
    except:
        old_pwd = None
    tempdir = mkdtemp()
    os.chdir(tempdir)
    try:
        os.symlink(REBUILDRNA_PATH + os.sep + 'histograms', 'histograms')
        os.symlink(REBUILDRNA_PATH + os.sep + 'pdb', 'pdb')
    except OSError:
        pass
    # remove strange residue names
    pdb_file = PDBFile(pdb_path=os.path.join(old_pwd, input_pdb))
    pdb_file._resname_3to1()
    pdb_file.save(os.path.join(tempdir, input_pdb))
    cmd = Popen([REBUILDRNA_PATH + os.sep + 'RebuildRNA',
        '-t', os.path.join(tempdir, input_pdb), '-o', tempdir])
    returncode = cmd.wait()
    out_name = tempdir + os.sep +\
        '.'.join(os.path.basename(input_pdb).split('.')[:-1]) + '_beforeCorrecting.pdb'
    os.rename(out_name, output_pdb)
    try:
        if old_pwd:
            os.chdir(old_pwd)
        rmtree(tempdir)
    except OSError:
        pass
    return returncode


def check_and_rebuild(input_pdb, output_pdb):
    """Check PDB representation and rebuild if necessary

    Arguments:
      * input_pdb = path to input PDB
      * output_pdb = path to output PDB

    Output:
      * True if rebuilding was necessary, False if not
    """
    f = open(input_pdb)
    input_pdb_string = f.read()
    f.close()
    if check_3_atom(input_pdb_string):
        rebuild_full_atom(input_pdb, output_pdb)
        return True
    else:
        copyfile(input_pdb, output_pdb)
        return False



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""rna_pdb_toolsx - a swiss army knife to manipulation of RNA pdb structures

Tricks:

   for i in *pdb; do rna_pdb_toolsx.py --get-rnapuzzle-ready $i >  ${i/.pdb/_rpr.pdb}; done

Usage::

   $ for i in *pdb; do rna_pdb_toolsx.py --delete A:46-56 $i > ../rpr_rm_loop/$i ; done

    $ rna_pdb_toolsx.py --get-seq *
    # BujnickiLab_RNApuzzle14_n01bound
    > A:1-61
    # BujnickiLab_RNApuzzle14_n02bound
    > A:1-61
    CGUUAGCCCAGGAAACUGGGCGGAAGUAAGGCCCAUUGCACUCCGGGCCUGAAGCAACGCG
    [...]

"""
import argparse
import textwrap
import os
import shutil
import sys
import tempfile
import glob
import os
from rna_tools.rna_tools_lib import edit_pdb, add_header, get_version, \
                          collapsed_view, fetch, fetch_ba, replace_chain, RNAStructure, \
                          select_pdb_fragment, sort_strings
from rna_tools.tools.rna_x3dna.rna_x3dna import x3DNA


def get_parser():
    version = os.path.basename(os.path.dirname(os.path.abspath(__file__))), get_version(__file__)
    version = version[1].strip()
    parser = argparse.ArgumentParser(
        description=__doc__, formatter_class=argparse.RawTextHelpFormatter)

    parser.add_argument('--version', help='', action='version', version=version)

    parser.add_argument('-r', '--report', help='get report',
                        action='store_true')

    parser.add_argument('--renum-atoms', help='renumber atoms, tested with --get-seq',
                        action='store_true')

    parser.add_argument('--renum-residues-dirty', help='',  action='store_true')

    parser.add_argument('--delete-anisou', help='remove files with ANISOU records, works with --inplace',
                        action='store_true')

    parser.add_argument('--fix', help='fix a PDB file, ! external program, pdbfixer used to fix missing atoms',
                        action='store_true')

    parser.add_argument('--to-mol2', help='fix a PDB file, ! external program, pdbfixer used to fix missing atoms',
                        action='store_true')

    parser.add_argument('--split-alt-locations', help='@todo',
                        action='store_true')

    parser.add_argument('-c', '--clean', help='get clean structure',
                        action='store_true')

    parser.add_argument('--is-pdb', help='check if a file is in the pdb format',
                        action='store_true')

    parser.add_argument('--is-nmr', help='check if a file is NMR-style multiple model pdb',
                        action='store_true')

    parser.add_argument('--nmr-dir', help='make NMR-style multiple model pdb file from a set of files \n\n' +
                        "  rna_pdb_toolsx.py --nmr-dir . 'cwc15_u5_fragments*.pdb' > ~/Desktop/cwc15-u5.pdb\n\n" +
                        "please use '' for pattern file recognition, this is a hack to deal with folders with\n"
                        "thousands of models, if you used only *.pdb then the terminal will complain that you\n"
                        "selected to many files.")

    parser.add_argument('--un-nmr', help='split NMR-style multiple model pdb files into individual models [biopython]',
                        action='store_true')

    parser.add_argument('--orgmode', help='get a structure in org-mode format <sick!>',
                        action='store_true')

    parser.add_argument('--get-chain', help='get chain, one or many, e.g, A, but now also ABC works')

    parser.add_argument('--fetch', action='store_true', help='fetch file from the PDB db')

    parser.add_argument('--fetch-ba', action='store_true',
                        help='fetch biological assembly from the PDB db')

    parser.add_argument('--get-seq', help='get seq', action='store_true')
    parser.add_argument('--hide-warnings', help='hide warnings, works with --get-chain, it hides warnings that given changes are not detected in a PDB file', action='store_true')

    parser.add_argument('--compact',
                        help=textwrap.dedent("""with --get-seq, get it in compact view'
$ rna_pdb_toolsx.py --get-seq --compact *.pdb
# 20_Bujnicki_1
ACCCGCAAGGCCGACGGCGCCGCCGCUGGUGCAAGUCCAGCCACGCUUCGGCGUGGGCGCUCAUGGGU # A:1-68
# 20_Bujnicki_2
ACCCGCAAGGCCGACGGCGCCGCCGCUGGUGCAAGUCCAGCCACGCUUCGGCGUGGGCGCUCAUGGGU # A:1-68
# 20_Bujnicki_3
ACCCGCAAGGCCGACGGCGCCGCCGCUGGUGCAAGUCCAGCCACGCUUCGGCGUGGGCGCUCAUGGGU # A:1-68
# 20_Bujnicki_4

"""), action='store_true')

    parser.add_argument('--get-ss', help='get secondary structure', action='store_true')

    parser.add_argument('--rosetta2generic', help='convert ROSETTA-like format to a generic pdb',
                        action='store_true')

    parser.add_argument('--get-rnapuzzle-ready',
                        help=textwrap.dedent("""get RNApuzzle ready (keep only standard atoms).'
Be default it does not renumber residues, use --renumber-residues
[requires BioPython]"""), action='store_true')

    parser.add_argument('--rpr', help='alias to get_rnapuzzle ready)',
                        action='store_true')

    parser.add_argument('--no-hr', help='do not insert the header into files',
                        action='store_true')

    parser.add_argument('--renumber-residues', help='by defult is false',
                        action='store_true')

    parser.add_argument('--dont-rename-chains',
                        help=textwrap.dedent("""used only with --get-rnapuzzle-ready.
By default:
   --get-rnapuzzle-ready rename chains from ABC.. to stop behavior switch on this option
"""),
                        action='store_true')

    parser.add_argument('--dont-fix-missing-atoms',
                        help="""used only with --get-rnapuzzle-ready""",
                        action='store_true')

    parser.add_argument('--dont-report-missing-atoms',
                        help="""used only with --get-rnapuzzle-ready""",
                        action='store_true')

    parser.add_argument('--collapsed-view', help='',
                        action='store_true')

    parser.add_argument('--cv', help='alias to collapsed_view',
                        action='store_true')

    parser.add_argument('-v', '--verbose', help='tell me more what you\'re doing, please!',
                        action='store_true')

    parser.add_argument('--replace-hetatm', help="replace 'HETATM' with 'ATOM' [tested only with --get-rnapuzzle-ready]",
                        action="store_true")

    parser.add_argument('--inplace', help=textwrap.dedent("""in place edit the file! [experimental,
only for get_rnapuzzle_ready, delete, --get-ss, --get-seq, --edit-pdb]"""),
                        action='store_true')

    parser.add_argument('--mutate', help=textwrap.dedent("""mutate residues,
e.g.,
      --mutate "A:1A+2A+3A+4A,B:1A"
to mutate to adenines the first four nucleotides of the chain A
and the first nucleotide of the chain B"""))

    parser.add_argument('--edit',
                        dest="edit",
                        default='',
                        help="edit 'A:6>B:200', 'A:2-7>B:2-7'")

    parser.add_argument('--rename-chain',
                        help="edit 'A>B' to rename chain A to chain B")

    parser.add_argument('--swap-chains', help='B>A, rename A to _, then B to A, then _ to B')

    parser.add_argument('--replace-chain',
                        default='',
                        help=textwrap.dedent("""a file PDB name with one chain that will be used to
replace the chain in the original PDB file,
the chain id in this file has to be the same with the chain id of the original chain"""))

    parser.add_argument('--delete',  # type="string",
                        dest="delete",
                        default='',
                        help="delete the selected fragment, e.g. A:10-16, or for more than one fragment --delete 'A:1-25+30-57'")

    parser.add_argument('--extract',  # type="string",
                        dest="extract",
                        default='',
                        help="extract the selected fragment, e.g. A:10-16, or for more than one fragment --extract 'A:1-25+30-57'")

    parser.add_argument('--extract-chain',
                         help="extract chain, e.g. A")


    parser.add_argument('--uniq', help=textwrap.dedent("""
rna_pdb_toolsx.py --get-seq --uniq '[:5]' --compact --chain-first * | sort
A:1-121        ACCUUGCGCAACUGGCGAAUCCUGGGGCUGCCGCCGGCAGUACCC...CA # rp13nc3295_min.out.1
A:1-123        ACCUUGCGCGACUGGCGAAUCCUGAAGCUGCUUUGAGCGGCUUCG...AG # rp13cp0016_min.out.1
A:1-123        ACCUUGCGCGACUGGCGAAUCCUGAAGCUGCUUUGAGCGGCUUCG...AG # zcp_6537608a_ALL-000001_AA
A:1-45 57-71   GGGUCGUGACUGGCGAACAGGUGGGAAACCACCGGGGAGCGACCCGCCGCCCGCCUGGGC # solution
"""))

    parser.add_argument('--chain-first', help="", action='store_true')
    parser.add_argument('--oneline', help="", action='store_true')
    parser.add_argument('--fasta',
                        help= textwrap.dedent("""with --get-seq, show sequences in fasta format,
can be combined with --compact (mind, chains will be separated with ' ' in one line)

$ rna_pdb_toolsx.py --get-seq --fasta --compact input/20_Bujnicki_1.pdb
> 20_Bujnicki_1
ACCCGCAAGGCCGACGGC GCCGCCGCUGGUGCAAGUCCAGCCACGCUUCGGCGUGGGCGCUCAUGGGU

"""), action='store_true')

    parser.add_argument('file', help='file', nargs='+')
    #parser.add_argument('outfile', help='outfile')
    return parser


# main
if __name__ == '__main__':
    # get version
    version = os.path.basename(os.path.dirname(os.path.abspath(__file__))), get_version(__file__)
    version = version[1].strip()

    # get parser and arguments
    parser = get_parser()

    args = parser.parse_args()

    # quick fix for one files vs file-s
    if list == type(args.file) and len(args.file) == 1:
        args.file = args.file[0]

    if args.report:
        s = RNAStructure(args.file)
        print(s.get_report)
        print(s.get_preview())
        print(s.get_info_chains())

    if args.clean:
        s = RNAStructure(args.file)
        s.decap_gtp()
        s.std_resn()
        s.remove_hydrogen()
        s.remove_ion()
        s.remove_water()
        s.renum_atoms()
        s.fix_O_in_UC()
        s.fix_op_atoms()
        # print s.get_preview()
        # s.write(args.outfile)
        if not args.no_hr:
            print(add_header(version))
        print(s.get_text())

    if args.get_seq:
        # quick fix - make a list on the spot
        if list != type(args.file):
            args.file = [args.file]
        ##################################
        analyzed = []
        for f in args.file:
            #####################################
            if args.uniq:
                subname = eval('f' + args.uniq)
                if subname in analyzed:
                    continue
                else:
                    analyzed.append(subname)
            ########
            s = RNAStructure(f)
            if not s.is_pdb():
                print('Error: Not a PDB file %s' % f)
                sys.exit(1)
            s.decap_gtp()
            s.std_resn()
            s.remove_hydrogen()
            s.remove_ion()
            s.remove_water()
            if args.renum_atoms:
                s.renum_atoms()
            s.fix_O_in_UC()
            s.fix_op_atoms()

            output = ''

            # with # is easier to grep this out
            if args.fasta:
                output += s.get_seq(compact=args.compact, chainfirst=args.chain_first, fasta=args.fasta, addfn=s.fn) + '\n'
            elif args.oneline:
                output += s.get_seq(compact=args.compact, chainfirst=args.chain_first).strip() + ' # '+ os.path.basename(f.replace('.pdb', '')) + '\n'
            else:
                output += '# ' + os.path.basename(f.replace('.pdb', '')) + '\n'
                output += s.get_seq(compact=args.compact, chainfirst=args.chain_first) + '\n'

            try:
                sys.stdout.write(output)
                sys.stdout.flush()
            except IOError:
                pass

    if args.get_ss:
        # quick fix - make a list on the spot
        if list != type(args.file):
            args.file = [args.file]
        ##################################
        for f in args.file:
            output = f + '\n'
            p = x3DNA(f)
            output += p.get_secstruc() + '\n'
            try:
                sys.stdout.write(output)
                sys.stdout.flush()
            except IOError:
                pass

    # getchain
    if args.get_chain:
        s = RNAStructure(args.file)
        ## s.std_resn()
        ## s.remove_hydrogen()
        ## s.remove_ion()
        ## s.remove_water()
        ## s.renum_atoms()
        ## s.fix_O_in_UC()
        ## s.fix_op_atoms()
        # print s.get_preview()
        warningmsg = ''
        for chain in list(args.get_chain):
            chain_txt = s.get_chain(chain)
            if not chain_txt.strip():
                warningmsg += 'Warning: Chain %s not detected!' % chain
            else:
                print(chain_txt)
        if not args.hide_warnings:
            if warningmsg:
                print(warningmsg)

    if args.rosetta2generic:
        s = RNAStructure(args.file)
        s.std_resn()
        s.remove_hydrogen()
        s.remove_ion()
        s.remove_water()
        s.fix_op_atoms()
        s.renum_atoms()
        # print s.get_preview()
        # s.write(args.outfile)
        if not args.no_hr:
            print(add_header(version))
        print(s.get_text())

    #rpr #--rpr
    if args.get_rnapuzzle_ready or args.rpr:
        # quick fix - make a list on the spot
        if list != type(args.file):
            args.file = [args.file]
        ##################################
        # progress bar only in --inplace mode!
        if args.inplace:
            import progressbar
            bar = progressbar.ProgressBar(max_value=len(args.file))
            bar.update(0)

        for c, f in enumerate(args.file):
            if args.verbose:
                print(f)
            if args.inplace:
                shutil.copy(f, f + '~')

            # keep previous edits
            previous_edits = []
            with open(f) as fx:
                for l in fx:
                    if l.startswith('HEADER --'):
                        previous_edits.append(l.strip())
            ######################

            s = RNAStructure(f)
            if args.replace_hetatm:
                s.replace_hetatm()
            s.decap_gtp()
            s.std_resn()
            s.remove_hydrogen()
            s.remove_ion()
            s.remove_water()
            s.fix_op_atoms()
            s.renum_atoms()
            s.shift_atom_names()
            s.prune_elements()

            rename_chains = False if args.dont_rename_chains else True

            report_missing_atoms = not args.dont_report_missing_atoms
            fix_missing_atom = not args.dont_fix_missing_atoms

            remarks = s.get_rnapuzzle_ready(args.renumber_residues, fix_missing_atoms=fix_missing_atom,
                                            rename_chains=rename_chains,
                                            report_missing_atoms=report_missing_atoms,
                                            verbose=args.verbose)

            if args.inplace:
                with open(f, 'w') as f:
                    if not args.no_hr:
                        f.write(add_header(version) + '\n')
                    if previous_edits:
                        f.write('\n'.join(previous_edits) + '\n')
                    if remarks:
                        f.write('\n'.join(remarks) + '\n')
                    f.write(s.get_text())

                # progress bar only in --inplace mode!
                bar.update(c)

            else:
                output = ''
                if not args.no_hr:
                    output += add_header(version) + '\n'
                if remarks:
                    output += '\n'.join(remarks) + '\n'
                output += s.get_text() + '\n'
                try:
                    sys.stdout.write(output)
                    sys.stdout.flush()
                except IOError:
                    pass
        # hmm... fix for problem with renumbering, i do renumbering
        # and i stop here
        # i'm not sure that this is perfect
        sys.exit(0)

    if args.renumber_residues:
        s = RNAStructure(args.file)
        s.remove_hydrogen()
        s.remove_ion()
        s.remove_water()
        s.get_rnapuzzle_ready(args.renumber_residues)
        s.renum_atoms()
        if not args.no_hr:
            print(add_header(version))
        print(s.get_text())

# args.renumber_resides_dirty
    if args.renum_residues_dirty:
        # quick fix - make a list on the spot
        if list != type(args.file):
            args.file = [args.file]
        ##################################
        for f in args.file:
            if args.inplace:
                shutil.copy(f, f + '~')

            s = RNAStructure(f)

            output = ''
            #if not args.no_hr:
            #    output += add_header(version) + '\n'
            #    output += 'HEADER --delete ' + args.delete + '\n'  # ' '.join(str(selection))
            c = 1
            old_resi = -1
            for l in s.lines:
                if l.startswith('ATOM') or l.startswith("HETATOM"):
                    resi = int(l[23:26].strip())
                    if resi != old_resi:
                        old_resi = resi
                        c += 1
                    # print(resi, c)
                    #resi = c
                    #if chain in selection:
                    #    if resi in selection[chain]:
                    #        continue  # print chain, resi
                    output += l[:23] + str(c).rjust(3) + l[26:] + '\n'
            # write: inplace
            if args.inplace:
                with open(f, 'w') as f:
                    f.write(output)
            else:  # write: to stdout
                try:
                    sys.stdout.write(output)
                    sys.stdout.flush()
                except IOError:
                    pass



    if args.delete:
        # quick fix - make a list on the spot
        if list != type(args.file):
            args.file = [args.file]
        ##################################
        for f in args.file:
            if args.inplace:
                shutil.copy(f, f + '~')

            selection = select_pdb_fragment(args.delete)
            s = RNAStructure(f)

            output = ''
            if not args.no_hr:
                output += add_header(version) + '\n'
                output += 'HEADER --delete ' + args.delete + '\n'  # ' '.join(str(selection))
            for l in s.lines:
                if l.startswith('ATOM'):
                    chain = l[21]
                    resi = int(l[23:26].strip())
                    if chain in selection:
                        if resi in selection[chain]:
                            continue  # print chain, resi
                    output += l + '\n'

            # write: inplace
            if args.inplace:
                with open(f, 'w') as f:
                    f.write(output)
            else:  # write: to stdout
                try:
                    sys.stdout.write(output)
                    sys.stdout.flush()
                except IOError:
                    pass

    if args.replace_chain:
        # quick fix - make a list on the spot
        if list != type(args.file):
            args.file = [args.file]
        ##################################
        for f in args.file:
            if args.inplace:
                shutil.copy(f, f + '~')

            # --replace_chain <file> without A:<file> it will be easier than --x "A:<file>"
            s = RNAStructure(args.replace_chain)
            chain_ids = (s.get_all_chain_ids())
            if len(chain_ids) > 1:
                raise Exception('There is more than one chain in the inserted PDB file. There should be only one chain, the one you want to insert to the PDB.')
            out = replace_chain(f, args.replace_chain, list(chain_ids)[0])
            print(out)

    if args.mutate:
        # quick fix - make a list on the spot
        if list != type(args.file):
            args.file = [args.file]
        ##################################
        try:
            from moderna import *
        except ImportError:
            raise Exception('To use this functionality please install ModeRNA, e.g. pip install moderna')

        for f in args.file:
            if args.inplace:
                shutil.copy(f, f + '~')  # create a backup copy if inplace

            # create a working copy of the main file
            ftf = tempfile.NamedTemporaryFile(delete=False).name  # f temp file
            shutil.copy(f, ftf)  # create a backup copy if inplace

            # go over each chain
            # rna_pdb_toolsx.py --mutate 'A:1CB:1G,A:1U+B:1A' CG_AB.pdb > ~/Desktop/a.pdb
            for m in args.mutate.split(','):  # A:1A, B:1A
                chain, resi_mutate_to = m.strip().split(':')  # A:1A+2C
                resi_mutate_to_list = resi_mutate_to.split('+')  # A:1A+2C

                model = load_model(f, chain)
                # go over each mutation in this chain
                for resi_mutate_to in resi_mutate_to_list:
                    resi = resi_mutate_to[:-1]
                    mutate_to = resi_mutate_to[-1]
                    if args.verbose:
                        print('Mutate', model[resi], 'to', mutate_to)
                    exchange_single_base(model[resi], mutate_to)

                # multi mutation in one chain
                tf = tempfile.NamedTemporaryFile(delete=False)
                model.write_pdb_file(tf.name)

                # work on the copy of f, ftf
                output = replace_chain(ftf, tf.name, chain)
                with open(ftf, 'w') as tmp:
                    tmp.write(output)

            # write: inplace
            if args.inplace:
                # ftf now is f, get ready for the final output
                shutil.copy(ftf, f)
            else:  # write: to stdout
                try:
                    sys.stdout.write(output)
                    sys.stdout.flush()
                except IOError:
                    pass


    if args.extract:
        # quick fix - make a list on the spot
        if list != type(args.file):
            args.file = [args.file]
        ##################################
        for f in args.file:
            if args.inplace:
                shutil.copy(f, f + '~')

            selection = select_pdb_fragment(args.extract)
            s = RNAStructure(f)

            output = ''
            if not args.no_hr:
                output += add_header(version) + '\n'
                output += 'HEADER extract ' + args.extract + '\n'  # ' '.join(str(selection))
            for l in s.lines:
                if l.startswith('ATOM'):
                    chain = l[21]
                    resi = int(l[23:26].strip())
                    if chain in selection:
                        if resi in selection[chain]:
                            # continue  # print chain, resi
                            output += l + '\n'

            # write: inplace
            if args.inplace:
                with open(f, 'w') as f:
                    f.write(output)
            else:  # write: to stdout
                try:
                    sys.stdout.write(output)
                    sys.stdout.flush()
                except IOError:
                    pass


    if args.extract_chain:
        # quick fix - make a list on the spot
        if list != type(args.file):
            args.file = [args.file]
        ##################################
        for f in args.file:
            if args.inplace:
                shutil.copy(f, f + '~')

            selection = select_pdb_fragment(args.extract)
            s = RNAStructure(f)

            output = ''
            if not args.no_hr:
                output += add_header(version) + '\n'
                output += 'HEADER extract ' + args.extract + '\n'  # ' '.join(str(selection))
            for l in s.lines:
                if l.startswith('ATOM') or l.startswith('TER') or l.startswith('HETATM'):
                    chain = l[21]
                    if chain in args.extract_chain:
                        output += l + '\n'

            # write: inplace
            if args.inplace:
                with open(f, 'w') as f:
                    f.write(output)
            else:  # write: to stdout
                try:
                    sys.stdout.write(output)
                    sys.stdout.flush()
                except IOError:
                    pass

    if args.is_pdb:
        s = RNAStructure(args.file)
        output = str(s.is_pdb())
        sys.stdout.write(output + '\n')

    if args.un_nmr:
        s = RNAStructure(args.file)
        str(s.un_nmr())

    if args.is_nmr:
        struc = RNAStructure(args.file)
        output = str(struc.is_nmr(args.verbose))
        sys.stdout.write(output + '\n')

    #edit
    if args.edit:
        if list != type(args.file):
            args.file = [args.file]
        ##################################
        for f in args.file:
            if args.verbose:
                print(f)
            if args.inplace:
                shutil.copy(f, f + '~')

            output = edit_pdb(f, args)

            if args.inplace:
                with open(f, 'w') as f:
                    f.write(output)
            else:  # write: to stdout
                try:
                    sys.stdout.write(output)
                    sys.stdout.flush()
                except IOError:
                    pass


    if args.fetch:
        fetch(args.file)

    if args.fetch_ba:
        fetch_ba(args.file)

    if args.collapsed_view or args.cv:
        collapsed_view(args)

    if args.delete_anisou:
        # quick fix - make a list on the spot
        if list != type(args.file):
            args.file = [args.file]
        ##################################
        for f in args.file:
            if args.inplace:
                shutil.copy(f, f + '~')

            s = RNAStructure(f)

            output = ''
            if not args.no_hr:
                output += add_header(version) + '\n'

            for l in s.lines:
                if l.startswith('ANISOU'):
                    continue
                else:
                    output += l + '\n'

            if args.inplace:
                with open(f, 'w') as f:
                    f.write(output)
            else:  # write: to stdout
                try:
                    sys.stdout.write(output)
                    sys.stdout.flush()
                except IOError:
                    pass


    if args.swap_chains:
        # quick fix - make a list on the spot
        if list != type(args.file):
            args.file = [args.file]
        ##################################
        for f in args.file:
            if args.inplace:
                shutil.copy(f, f + '~')
            # rename_chain 'A>B'
            s = RNAStructure(f)
            output = ''
            if not args.no_hr:
                output += add_header(version) + '\n'

            chain_id_old, chain_id_new = args.swap_chains.split('>')
            if chain_id_old == chain_id_new:
                output = open(f).read() # return the file ;-) itself unchanged
            else:
                s.rename_chain(chain_id_new, '_')
                s.rename_chain(chain_id_old, chain_id_new)
                output += s.rename_chain('_', chain_id_old)

            if args.inplace:
                with open(f, 'w') as f:
                    f.write(output)
            else:  # write: to stdout
                try:
                    sys.stdout.write(output)
                    sys.stdout.flush()
                except IOError:
                    pass

    if args.rename_chain:
        # quick fix - make a list on the spot
        if list != type(args.file):
            args.file = [args.file]
        ##################################
        for f in args.file:
            if args.inplace:
                shutil.copy(f, f + '~')
            # rename_chain 'A>B'
            s = RNAStructure(f)
            chain_id_old, chain_id_new = args.rename_chain.split('>')
            output = ''
            if not args.no_hr:
                output += add_header(version) + '\n'
            output += s.rename_chain(chain_id_old, chain_id_new)
            if args.inplace:
                with open(f, 'w') as f:
                    f.write(output)
            else:  # write: to stdout
                try:
                    sys.stdout.write(output)
                    sys.stdout.flush()
                except IOError:
                    pass


    if args.split_alt_locations:
        # quick fix - make a list on the spot
        if list != type(args.file):
            args.file = [args.file]
        ##################################
        for f in args.file:
            if args.inplace:
                shutil.copy(f, f + '~')

            #s = RNAStructure(f)
            s = open(f)
            output = ''
            if not args.no_hr:
                output += add_header(version) + '\n'

            # First, collect all alternative locations.
            alts = set()
            for l in s:
                if l.startswith('ATOM'):
                    if l[16].strip():
                        alts.add(l[16])
            s.close()

            if args.verbose:
                print('alts:', alts)

            for index, alt in enumerate(alts):
                output += 'MODEL %i' % (index + 1)
                s = open(f)
                for l in s:
                    if l.startswith('ATOM') or l.startswith('HETATM'):
                        # if empty, then print this line
                        if l[16] == ' ':
                            output += l
                        if l[16] == alt:
                            output += l
                    else:
                        output += l
                output += 'ENDMDL\n'
                s.close()

            if args.inplace:
                with open(f, 'w') as f:
                    f.write(output)
            else:  # write: to stdout
                try:
                    sys.stdout.write(output)
                    sys.stdout.flush()
                except IOError:
                    pass

    if args.orgmode:
        if args.inplace:
            shutil.copy(args.file, args.file + '~')
        s = RNAStructure(args.file)
        s.decap_gtp()
        s.std_resn()
        s.remove_hydrogen()
        s.remove_ion()
        s.remove_water()
        s.fix_op_atoms()
        s.renum_atoms()
        s.shift_atom_names()
        s.prune_elements()
        # print s.get_preview()
        # s.write(args.outfile)
        # for l in s.lines:
        #    print l

        remarks = s.get_rnapuzzle_ready(
            args.renumber_residues, fix_missing_atoms=True, rename_chains=True, verbose=args.verbose)

        with open(args.file + '~', 'w') as f:
            if not args.no_hr:
                f.write(add_header(version) + '\n')

            f.write('\n'.join(remarks) + '\n')
            f.write(s.get_text())

        try:
            from Bio import PDB
            from Bio.PDB import PDBIO
            import warnings
            warnings.filterwarnings('ignore', '.*Invalid or missing.*',)
            warnings.filterwarnings('ignore', '.*with given element *',)
        except:
            sys.exit('Error: Install biopython to use this function (pip biopython)')

        parser = PDB.PDBParser()
        struct = parser.get_structure('', args.file + '~')
        model = struct[0]

        # chains [['A', 'seq', [residues]]]
        chains = []
        for c in model.get_list():
            seq = ''
            chain = []
            for r in c:
                chain.append(str(r.get_resname().strip()) + str(r.id[1]))
                seq += r.get_resname().strip()
            chains.append([c.id, seq, chain])

        t = []
        #[['A', 'CCGCCGCGCCAUGCCUGUGGCGG', ['C1', 'C2', 'G3', 'C4', 'C5', 'G6', 'C7', 'G8', 'C9', 'C10', 'A11', 'U12', 'G13', 'C14', 'C15', 'U16', 'G17', 'U18', 'G19', 'G20', 'C21', 'G22', 'G23']], ['B', 'CCGCCGCGCCAUGCCUGUGGCGG', ['C1', 'C2', 'G3', 'C4', 'C5', 'G6', 'C7', 'G8', 'C9', 'C10', 'A11', 'U12', 'G13', 'C14', 'C15', 'U16', 'G17', 'U18', 'G19', 'G20', 'C21', 'G22', 'G23']]]
        for c in chains:
            t.append('* ' + c[0] + ':' + c[2][0][1:] + '-' + c[2][-1][1:] + ' ' + c[1])
            for r in c[2]:
                t.append('** ' + c[0] + ':' + r)
        print('\n'.join(t))

    if args.fix:
        # quick fix - make a list on the spot
        if list != type(args.file):
            args.file = [args.file]
        ##################################
        for f in args.file:
            cmd = 'pdbfixer ' + f + ' --add-atoms all --add-residues'
            print(cmd)
            os.system(cmd)
            if args.inplace:
                shutil.move("output.pdb", f)
            else:
                shutil.move("output.pdb", f.replace('.pdb', '_fx.pdb'))
                            
    if args.to_mol2:
        # quick fix - make a list on the spot
        if list != type(args.file):
            args.file = [args.file]
        ##################################
        for f in args.file:
            cmd = 'obabel -i pdb ' + f + ' -o mol2 -O ' + f.replace('.pdb', '.mol2')
            print(cmd)
            os.system(cmd)


    if args.nmr_dir:
        files = sort_strings(glob.glob(args.nmr_dir + '/' + args.file))

        c = 1

        for f in files:
            #s = RNAStructure(f)
            print("MODEL        " + str(c))
            # at some point I could use RNAStructure for this
            print(open(f).read())
            #print(s.get_text(add_end=False))
            print('ENDMDL')
            c += 1
        print('END')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from .PDBFile import PDBFile


class RosettaPDBFile(PDBFile):
    """Class for preparing PDB files for ROSETTA/FARNA
    """
    def make_rna_rosetta_ready(self):
        """ROSETTA has some special requirements for PDB files.
    
        Arguments:
          * pdb_string = contents of PDB file as a string
    
        Output:
          * new PDB returned as a string
        """
        #self.check_and_get_first_model()
        self.resname_check_and_3to1()
        #self.terminate_chains()
        #self.remove_short_chains()
        result = []
        for l in self.pdb_lines:
            ## fix G__ -> __G
            if l.startswith('ATOM') and l[17] in ('A', 'U', 'C', 'G'):
                l = l[:16] + '   ' + l[17] + ' ' + l[21:]
            ##

            if l.startswith('ATOM') and l[17] in ('A', 'U', 'C', 'G'):
                line = l.replace('\'', '*')
                result.append(line[:18] + 'r' + line[19:])

            if l.startswith('ATOM') and l[19] in ('A', 'U', 'C', 'G'):
                line = l.replace('\'', '*')
                result.append(line[:18] + 'r' + line[19:])
            else:
                if not l.startswith('HETATM'):
                    result.append(l)
        self._apply_fix('make_rna_rosetta_ready', '\n'.join(result), result)
    
    
    def make_rna_non_rosetta(self):
        """Reverses changes required for ROSETTA, especially X -> rX.
    
        Arguments:
          * pdb_string = contents of PDB file as a string
    
        Output:
          * new PDB returned as a string
        """
        result = []
        for l in self.pdb_lines:
            if l.startswith('ATOM') and l[19] in ('A', 'U', 'C', 'G'):
                line = l.replace('\'', '*')
                result.append(line[:18] + ' ' + line[19:])
            else:
                result.append(l)
        self._apply_fix('make_rna_non_rosetta_ready', '\n'.join(result), result)


#!/usr/bin/env python

"""rna_align_find_core.py
"""

import rna_tools.tools.rna_alignment.rna_alignment as ra
import argparse

def get_parser():
    parser = argparse.ArgumentParser(description=__doc__, formatter_class=argparse.RawDescriptionHelpFormatter)
    parser.add_argument('file', help="fasta seq")
    return parser

if __name__ == '__main__':
    # for debugging #
    #fn = 'test_data/RF00167.stockholm.sto'
    #ids = ['AL591975.1/251136-251218','CP000721.1/2204691-2204778']#ACCL02000010.1/116901-116991']
    parser = get_parser()
    args = parser.parse_args()
    fn = args.file

    a = ra.RNAalignment(fn)
    print(a.find_core())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""

"""
from __future__ import print_function

import argparse
from Bio import SeqIO


def get_parser():
    parser = argparse.ArgumentParser(
        description=__doc__, formatter_class=argparse.RawDescriptionHelpFormatter)

    #parser.add_argument('-', "--", help="", default="")

    parser.add_argument("-v", "--verbose",
                        action="store_true", help="be verbose")
    parser.add_argument("conserv", help="consigns file", default="")
    parser.add_argument("alignment", help="", default="")
    parser.add_argument("chain", help="", default="A")
    return parser


if __name__ == '__main__':
    parser = get_parser()
    args = parser.parse_args()

    consens = open(args.conserv).readline().strip().split(',')[1:]
    #print('Consensus:', consens)
    chain = args.chain

    for record in SeqIO.parse(args.alignment, "fasta"):
        header = record.id
        seq = record.seq
        break

    c = 1
    dat = []
    txt = ''
    seqq = ''
    bar = ''
    color = ''
    for s, cons in zip(seq, consens):
        if s != '-':
            dat.append([c, cons])
            seqq += s
            if float(cons) == 100:
                bar += '*'
                color = 'red'
            elif float(cons) > 90:
                color = 'red'
                bar += '▇'
            elif float(cons) > 75:
                color = 'orange'
                bar += '▆'
            elif float(cons) > 60:
                color = 'brown'
                bar += '▅'
            elif float(cons) > 45:
                color = 'yellow'
                bar += '▄'
            elif float(cons) > 30:
                color = 'gray30'
                bar += '▂'
            else:
                color = 'gray'
                bar += '▁'
            if not color:
              print(color, c, chain, cons)
            l = 'color %s, resi %s and chain %s; #%s' % (color, c, chain, cons)
            print(l)
            txt += l
            #print(c, s, cons)
            c += 1
    print('# ' + seqq)
    print('# ' + bar)



from .simrna_trajectory import SimRNATrajectory

if __name__ == '__main__':
    fn_trafl = 'test_data/mini.trafl'
    fn = (line for line in open(fn_trafl).xreadlines())
    c = 0
    while 1:
        try:
            header = fn.next().strip()
        except StopIteration:  # not nice
            break
        c += 1
        coords = fn.next().strip()

        traj = SimRNATrajectory()
        traj.load_from_string(c, header + '\n' + coords)
        f = traj.frames[0]
        print(f.header)
        idx = range(13, 16 + 1) + range(34, 36 + 1) + range(45, 49 + 1)
        for r in [f.residues[i] for i in idx]:
            # print 'atoms of ', r, r.get_atoms_txt()
            print(r.get_atoms_txt(),)
        print



#!/usr/bin/python

import subprocess

cmd="/home/oge/bin/lx24-amd64/qstat -u '*' | grep -v 'hqw' "
out = subprocess.getoutput(cmd)
cc = 0
for l in out.split('\n'):
    if l:
        c = l.split()[-1]
        try:
            cc += int(c)
        except ValueError:
            pass
print((cc, cc/1000.0, -(1000 - cc)))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""This is a quick and dirty method of comparison two RNA structures (stored in pdb files).
It measures the distance between the relevan atoms (C4') for nucleotides defined as "x" in the
sequence alignment.

author: F. Stefaniak, modified by A. Zyla,  supervision of mmagnus
"""
from __future__ import print_function
from Bio.PDB import PDBParser
from scipy.spatial import distance
from Bio.SeqRecord import SeqRecord
from Bio import SeqIO
import logging
import argparse
import numpy as np
import matplotlib.pyplot as plt

# logger
logger = logging.getLogger()
handler = logging.StreamHandler()
logger.addHandler(handler)


def get_seq(alignfn, seqid):
    """Get seq from an alignment with gaps.

    Args:

       alignfn (str): a path to an alignment
       seqid   (str): seq id in an alignment

    Usage::

        >>> get_seq('test_data/ALN_OBJ1_OBJ2.fa', 'obj1')
        SeqRecord(seq=SeqRecord(seq=Seq('GUUCAG-------------------UGAC-', SingleLetterAlphabet()), id='obj1', name='obj1', description='obj1', dbxrefs=[]), id='<unknown id>', name='<unknown name>', description='<unknown description>', dbxrefs=[])

    Returns:
         SeqRecord
    """
    # alignment = AlignIO.read(alignfn, 'fasta')
    alignment = SeqIO.index(alignfn, 'fasta')
    # print SeqRecord(alignment[seqid])
    sequence = SeqRecord(alignment[seqid])
    return sequence


def open_pdb(pdbfn):
    """Open pdb with Biopython.

    Args:
       pdbfn1 (str): a path to a pdb structure

    Returns:
       PDB Biopython object: with a pdb structure

    """
    parser = PDBParser()
    return parser.get_structure('', pdbfn)


def find_core(seq_with_gaps1, seq_with_gaps2):
    """.

    Args:
        seq_with_gaps1 (str): a sequence 1 from the alignment
        seq_with_gaps1 (str): a sequence 2 from the alignment

    Usage::

        >>> find_core('GUUCAG-------------------UGAC-', 'CUUCGCAGCCAUUGCACUCCGGCUGCGAUG')
        'xxxxxx-------------------xxxx-'


    Returns:
        core="xxxxxx-------------------xxxx-"

    """
    core = "".join(["x" if (a != '-' and b != '-') else "-" for (a, b)
                    in zip(seq_with_gaps1, seq_with_gaps2)])
    return core


def map_coords_atom(structure):
    """.

        Args:
        structure (pdb): PDB Biopython object: with a pdb structure

        Returns:
            struct1dict: a list of coords for atoms
            structure1realNumber: a list of residues
        """

    resNumber = 0
    struct1dict = {}
    structure1realNumbers = {}

    for res in structure.get_residues():
        # print res
        for atom in res:
            name, coord = atom.name.strip(), atom.coord
            # G C5' [-15.50800037  -7.05600023  13.91800022]
            if name == atomToCompare:
                # print name, coord
                struct1dict[resNumber] = coord
                structure1realNumbers[resNumber] = res.get_full_id()[3][1]

        resNumber += 1
    return struct1dict,  structure1realNumbers


def get_parser():
    parser = argparse.ArgumentParser(description=__doc__,
                                     formatter_class=argparse.RawDescriptionHelpFormatter)
    parser.add_argument("-v", "--verbose", help="increase output verbosity",
                        action="store_true")
    parser.add_argument("seqid1", help="seq1 id in the alignemnt")
    parser.add_argument("seqid2", help="seq2 id in the alignemnt")
    parser.add_argument("alignfn", help="alignemnt in the Fasta format")
    parser.add_argument("pdbfn1", help="pdb file1")
    parser.add_argument("pdbfn2", help="pdb file2")
    return parser


# main
if __name__ == '__main__':
    import doctest
    doctest.testmod()

    args = get_parser().parse_args()

    if args.verbose:
        logger.setLevel(logging.INFO)

    # if not args.outfn:
        # args.outfn = args.pdbfn.replace('.pdb', '_out.pdb')

    seq_with_gaps1 = get_seq(args.alignfn, args.seqid1)
    seq_with_gaps2 = get_seq(args.alignfn, args.seqid2)
    pdb1 = open_pdb(args.pdbfn1)
    pdb2 = open_pdb(args.pdbfn2)
    core = find_core(seq_with_gaps1, seq_with_gaps2)
    atomToCompare = "C4'"
    sep = '\t'

    structure1 = pdb1
    structure2 = pdb2

    struct1dict, structure1realNumbers = map_coords_atom(pdb1)
    struct2dict, structure2realNumbers = map_coords_atom(pdb2)

    stats = []
    stats.append(["res1", "res2", "distance [A]"])

    resNumber = 0
    seq1number = -1
    seq2number = -1

    for char in core:
        # local sequence numbering
        if seq_with_gaps1[resNumber] != '-':
            seq1number += 1
            # print "seq1", seq1number, seq1[resNumber]
        if seq_with_gaps2[resNumber] != '-':
            seq2number += 1
            # print "seq2", seq2number, seq2[resNumber]

        # alignment checking (iksy)
        if char == 'x':
            vect1 = struct1dict[seq1number]
            vect2 = struct2dict[seq2number]
            stats.append([str(structure1realNumbers[seq1number]),
                          str(structure2realNumbers[seq2number]),
                          str(distance.euclidean(vect1, vect2))])
            # print vect1,vect2

        resNumber += 1

    # struc = renumber(seq_with_gaps, pdb, args.residue_index_start)
    # write_struc(struc, args.outfn)
    list_res=[]
    for i in stats:
        print(sep.join(i))
        table=(sep.join(i))
        list_res.append(i)
    #print (list_res)
    res_matrix = np.array(list_res[1:])

    '''
    Creating a plot

    '''

    new_resi = list_res[1:]
    new_resis = []

    for i in new_resi:
        #print (j)
        new_resis.append(str(i[0]) + '/' + str(i[1]))


    #print (new_resis)

    list2_matrix= new_resis

    list2_matrix1 = map(float, list(res_matrix[:,2]))
    #print (list2_matrix1)

    plt.bar(list2_matrix,list2_matrix1,facecolor='pink' )

    plt.suptitle('Distance between C4 atoms of residues')
    plt.ylabel("distance [A]")
    plt.xlabel('Nr of residue')
    plt.show()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""rna_ex2x.py - analyze an evolutionary coupling file.

Files can be downloaded from https://marks.hms.harvard.edu/ev_rna/, e.g. RF00167.EC.interaction.csv

``--pairs``::

    $ rna_ex2x.py RF00167.EC.interaction_LbyN.csv --pairs
    [18, 78],[31, 39],[21, 75],[30, 40],[28, 42],[27, 43],[59, 67],[54, 72],[57, 69],[25, 45],[29, 41],[17, 79],[26, 44],[16, 80],[14, 82],[19, 77],[55, 71],[
        15, 81],[34, 63],[56, 70],[58, 68],[35, 63],[26, 45],[35, 64],[32, 39],[54, 73],[24, 74],[16, 82],[24, 45],[24, 43],[32, 36],[25, 48],[48, 82],[36, 48],
"""
from __future__ import print_function

import csv
import pandas
import sys
import argparse


def get_parser():
    """Get parser."""
    parser = argparse.ArgumentParser(
        description=__doc__, formatter_class=argparse.RawDescriptionHelpFormatter)

    parser.add_argument("--sep", default=r",", help="separator")
    parser.add_argument("--chain", default="A", help="chain")
    # parser.add_argument("--offset", default=0)
    parser.add_argument('interaction_fn', help='interaction file')
    parser.add_argument('--ec-pairs',  action='store_true')
    parser.add_argument('--ss-pairs',  help="file with secondary structure base pairs")
    parser.add_argument('--pairs-delta', help="delta: ec-bp - ss-paris", action="store_true")
    return parser


if __name__ == '__main__':
    parser = get_parser()
    args = parser.parse_args()

    df = pandas.read_csv(args.interaction_fn, sep=",")
    pairs = []
    for index, row in df.iterrows():
        if str(row['pdb_resid1']) != 'nan':
            # ? uggly hack to 78.0 -> 78
            if not args.ec_pairs:
                print('d:' + args.chain + str(row['pdb_resid1']).replace('.0', '') +
                      '-' + args.chain + str(row['pdb_resid2']).replace('.0', '') + ' < 100 1')
            else:
                pairs.append([int(row['pdb_resid1']), int(row['pdb_resid2'])])
                pairs.sort()

                # this also worked, but it was an ugly print
                # print('[' + str(row['pdb_resid1']).replace('.0', '') + ', ' +
                #    str(row['pdb_resid2']).replace('.0', '') + ']', end=',')

    if args.ec_pairs:
        print('pairs:')
        print(pairs)

    if args.ss_pairs:
        ssbps = eval(open(args.ss_pairs).read().strip())
        ssbps.sort()
        print('ssbps:')
        print(ssbps)

    if args.pairs_delta:
        # go ever dca and keep if not in ss
        pairsnonss = []
        for pair in pairs:
            if pair in ssbps:
                pass
            else:
                pairsnonss.append(pair)
        print('# of ec_paris:', len(pairs))
        print('# of ssbps   :', len(ssbps))
        print('dalta#       :', len(pairsnonss))
        print(pairsnonss)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Future: the script will be integrated into RNAalignment class
"""

from __future__ import print_function
import rna_tools.tools.rna_alignment.rna_alignment as ra
import argparse
import subprocess


def get_parser():
    parser = argparse.ArgumentParser(description=__doc__, formatter_class=argparse.RawDescriptionHelpFormatter)
    parser.add_argument('file', help="fasta seq")
    parser.add_argument("-v", "--verbose", default=True,
                        action="store_true", help="be verbose")
    return parser


def exe(cmd, verbose):
    if verbose:
        print(cmd)
    o = subprocess.Popen(
        cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    out = o.stdout.read().strip().decode()
    err = o.stderr.read().strip().decode()
    return out, err


def r2r(alignment, verbose):
    cons =  alignment + '.cons'
    pdf =  alignment.replace('.stk', '').replace('.sto', '') + '.pdf'
    svg =  alignment.replace('.stk', '').replace('.sto', '') + '.svg'
    cmd = 'r2r --GSC-weighted-consensus ' + alignment + ' ' + cons + ' 3 0.97 0.9 0.75 4 0.97 0.9 0.75 0.5 0.1'
    out, err = exe(cmd, verbose)
    if verbose: print(out)
    if err:
        print(err)

    cmd = 'r2r ' + cons + ' ' + output
    out, err = exe(cmd, verbose)
    if err:
        print(err)
    if verbose: print(out)

    cmd = 'r2r ' + cons + ' ' + output
    out, err = exe(cmd, verbose)
    if err:
        print(err)
    if verbose: print(out)


if __name__ == '__main__':
    # for debugging #
    #fn = 'test_data/RF00167.stockholm.sto'
    #ids = ['AL591975.1/251136-251218','CP000721.1/2204691-2204778']#ACCL02000010.1/116901-116991']
    parser = get_parser()
    args = parser.parse_args()

    r2r(args.file, args.verbose)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Convert a secondary structure with a pk to the SimRNA format::

       rna_convert_pseudoknot_formats git:(master) ✗ python rna_ss_pk_to_simrna.py test_data/ss_with_pk.ss
       ((((([[[[[[)))))........(.((....(]]]]]].)..(((. .)))...)).)

       (((((......)))))........(.((....(.......)..(((. .)))...)).)
       .....((((((......................))))))........ ...........
"""

import argparse

SECOND_PK_CHAR = ['{', '}']

def is_pk(ss):
    if ss.find('[') > -1:
        return True
    if ss.find(']') > -1:
        return True

def get_multiple_lines(ss):
    ss_n = ''
    ss_pk = ''
    for i in ss.strip():
        if i == '[':
            j = '('
            i = '.'
        elif i == ']':
            j = ')'
            i = '.'
        elif i == ' ': # not space
            j = ' '
        else:
            j = '.'

        ss_n += i
        ss_pk += j

    return ss_n + '\n' + ss_pk

def get_parser():
    parser = argparse.ArgumentParser()
    parser.add_argument('file', help='file')
    return parser

if __name__ == '__main__':
    parser = get_parser()
    args = parser.parse_args()
    if args.file:
        ss = open(args.file).readline()
        print(ss)
        if is_pk(ss):
            print((get_multiple_lines(ss)))
        else:
            print(ss)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""SimRNA cluster - a simple wrapper around clustering of SimRNA.

By default: cluster 1% and give 5 clusters

.. warning :: clustering has to be visiable in your shell by the script!"""

from __future__ import print_function
from rna_tools.tools.simrna_trajectory.simrna_trajectory import SimRNATrajectory
import argparse
import os


def get_parser():
    parser = argparse.ArgumentParser(
        description=__doc__, formatter_class=argparse.RawDescriptionHelpFormatter)
    parser.add_argument('trafl', help="SimRNA trafl file")
    return parser


if __name__ == '__main__':
    parser = get_parser()
    args = parser.parse_args()

    fn = args.trafl
    # 12.5
    # How to get the length?
    t = SimRNATrajectory()
    t.load_from_file(fn, only_first_frame=True)

    # print len(t) # get length of trajectory (# of frames)
    print('len of the first frame:', len(t[0]))  # get length of 1 frames (# of residues)

    # fraction of lowest energy frames to clustering: 0.01
    # rmsd thrs for clustering 0.1*seq_lenght which is: 12.5
    cutoff = 0.1 * len(t[0])
    cmd = "clustering " + fn + " 0.01 " + str(cutoff) + " | tee clustering.log"
    print(cmd)
    os.system(cmd)



#!/usr/bin/env python
import math


def draw_circle(x, y, z, r=8.0, cr=1.0, cg=0.4, cb=0.8, w=2.0):
    """
    Create a CGO circle

    PARAMS
          x, y, z
            X, Y and Z coordinates of the origin

          r
            Radius of the circle

          cr, cg, cb
            Color triplet, [r,g,b] where r,g,b are all [0.0,1.0].

          w
            Line width of the circle

    RETURNS
          the CGO object (it also loads it into PyMOL, too).

    """
    x = float(x)
    y = float(y)
    z = float(z)
    r = abs(float(r))
    cr = abs(float(cr))
    cg = abs(float(cg))
    cb = abs(float(cb))
    w = float(w)

    obj = [BEGIN, LINES, COLOR, cr, cg, cb]
    for i in range(180):
        obj.append(VERTEX)
        obj.append(r * math.cos(i) + x)
        obj.append(r * math.sin(i) + y)
        obj.append(z)
        obj.append(VERTEX)
        obj.append(r * math.cos(i + 0.1) + x)
        obj.append(r * math.sin(i + 0.1) + y)
        obj.append(z)
    obj.append(END)

    cName = cmd.get_unused_name("circle_")
    cmd.load_cgo(obj, cName)
    cmd.set("cgo_line_width", w, cName)
    return obj


def draw_circle_selection(selName, r=None, cr=1.0, cg=0.4, cb=0.8, w=2.0):
    """
    circleSelection -- draws a cgo circle around a given selection or object

    PARAMS
          selName
            Name of the thing to encircle.

          r
            Radius of circle.
            DEFAULT: This cript automatically defines the radius for you.  If
            you select one atom and the resultant circle is too small, then
            you can override the script's calculation of r and specify your own.

          cr, cg, cb
            red, green and blue coloring, each a value in the range [0.0, 1.0]

    RETURNS
          The circle object.

    """
    ((minX, minY, minZ), (maxX, maxY, maxZ)) = cmd.get_extent(selName)

    if r == None:
        r = max([maxX - minX, maxY - minY, maxZ - minZ])

    stored.coords = []
    cmd.iterate_state(1, selName, "stored.coords.append([x,y,z])")
    l = len(stored.coords)

    centerX = sum([x[0] for x in stored.coords]) / l
    centerY = sum([x[1] for x in stored.coords]) / l
    centerZ = sum([x[2] for x in stored.coords]) / l

    return cgoCircle(centerX, centerY, centerZ, r, cr, cg, cb, w)


def draw_dist(x1, y1, z1, x2, y2, z2):
    """
    draw_dist(54.729, 28.9375, 41.421, 55.342, 35.3605, 42.745)

    https://sourceforge.net/p/pymol/mailman/message/25795427/
    """
    cmd.pseudoatom('pt1', pos=[x1, y1, z1])
    cmd.pseudoatom('pt2', pos=[x2, y2, z2])
    cmd.distance('pt1-pt2', 'pt1', 'pt2')


def draw_dists(interactions):  # l=([1,2], [3,4])
    for i in interactions:
        a = "////" + str(i[0]) + "/C2"
        b = "////" + str(i[1]) + "/C2"
        print(i[0], i[1], cmd.distance('d' + str(i[0]) + '-' +
                                       str(i[1]), "(" + a + ")", "(" + b + ")"))  # mode, 4))


def draw_vector(x1, y1, z1, x2, y2, z2):
    """https://pymolwiki.org/index.php/CGOCylinder"""
    radius = 0.1
    r1, g1, b1 = 0, 0, 1  # color (blue)
    r2, g2, b2 = 1, 0, 0  # color (red)
    cmd.load_cgo([9.0, x1, y1, z1, x2, y2, z2, radius, r1, g1, b1, r2, g2, b2], "vector")


try:
    import pymol
    from pymol.cgo import *
except:
    print('PyMOL (Python library is missing')
else:
    cmd.extend("draw_vector", draw_vector)
    cmd.extend("draw_dist", draw_dist)
    cmd.extend("draw_circle", draw_circle)
    cmd.extend("draw_circle_selection", draw_circle_selection)
    cmd.extend('draw_dists', draw_dists)



SIMRNA_DATA_PATH = None
RCHIE_PATH = None
QRNAS_PATH = None
VARNA_PATH = None
VARNA_JAR_NAME = None
RFAM_DB_PATH = None  # path to Rfam.cm
CONTEXTFOLD_PATH = None
CPUS_CLUSTER = 1000
DIFF_TOOL = "diff"

RNA_ROSETTA_RUN_ROOT_DIR_MODELING = "/home/magnus/rosetta-runs"
RNA_ROSETTA_NSTRUC = 10000

EASY_CAT_PATH = ""
RNASTRUCTURE_PATH = ""
ENTRNA_PATH = ''

import os
try:
    PATH = os.environ['RNA_TOOLS_PATH']
except KeyError:
    print ('Set up RNA_TOOLS_PATH, see Installation note')
    pass
else:
    QRNAS_PATH = os.getenv('QRNAS_PATH', PATH + '/opt/qrnas/')

try:
    from rna_tools.rna_tools_config_local import *
except ImportError:
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""rna_dca_mapping.py

This function is divided into three parts (and panels in the output):

1. map interactions on the gapped sequence,
2. map interactions on the sequence without gaps,
3. map interactions on the sequence that might have insertions (insertions are marked with lower-case letters).

Gseq and Seq are files in the Fasta format with a header, sequence, and secondary structure. Warning: the script is not tested (yet) with secondary structure with pseudoknots. It should work as well, but it is not tested yet. Secondary structures should be defined in the Vienna  ("dot-bracket") notation.

`--noshort` will remove interactions that the distance between them is less than 6. For example, an interaction like [1,3] will be removed.

`--noss` will remove all DCA interactions that come because of secondary structure::

   (..)
   [1,4] DCA interaction will be removed.

Q: your interactions start from 0 or 1?

For a DCA interaction file, ' ' is used for separator at the moment.

Required:  rna_tools.SecondaryStructure to parse secondary structures in the Vienna format (dot-bracket notation).

Example::

    [mm] tpp$ git:(master) ✗ rna_dca_mapping.py --gseq 2gdi_gapped_RmExtraPart_RedAligment.fa --seq 2gdi.fa --dca tpp.ec.txt_52_sselected.csv --noshort --offset 10 --noss
    interactions:
    [(5, 91), (6, 90), (7, 89), (8, 88), (9, 87), (10, 53), (11, 52), (12, 51), (13, 50), (15, 48), (16, 30), (17, 29), (18, 28), (18, 71), (19, 27), (20, 21), (20, 26), (21, 22), (21, 23), (21, 25), (22, 23), (22, 24), (23, 24), (24, 25), (26, 27), (36, 47), (37, 46), (37, 47), (38, 45), (39, 40), (39, 44), (40, 41), (40, 43), (41, 42), (42, 43), (43, 44), (47, 48), (54, 55), (55, 56), (56, 57), (60, 83), (61, 82), (63, 64), (63, 65), (65, 78), (66, 77), (67, 76), (68, 75), (69, 72), (69, 73), (69, 74), (72, 74)]
    pairs [[6, 92], [7, 91], [8, 90], [9, 89], [10, 88], [11, 54], [12, 53], [13, 52], [14, 51], [16, 38], [17, 31], [18, 30], [19, 29], [21, 28], [22, 27], [23, 26], [60, 85], [61, 84], [62, 83], [66, 79], [67, 78], [68, 77], [69, 76]]
    UNMAPPED SCORES ////////////////////////////////////////////////////////////////////////////////////////////////////////
    123456789112345678921234567893123456789412345678951234567896123456789712345678981234567899123456789
    -----GGACUCGGGGUGCCCUUCUUGAAGGCUGAGAAA----------UACCCGUAUCACCUGAUCUGGAUAAUGCCAGCGUAGGGAAGUUC------------
    -----(((((((((.((((.(((..))))))......)----------..)))).....(((...((((......))))...)))..)))))------------
                   x                                x                                                         [16, 49]
                      x                                                    x                                  [19, 72]
                       x       x                                                                              [20, 28]
                        x     x                                                                               [21, 27]
    MAPPED SCORES //////////////////////////////////////////////////////////////////////////////////////////////////////////
    Mapped Interactions:
    [[1, 77], [2, 76], [3, 75], [4, 74], [5, 73], [6, 39], [7, 38], [8, 37], [9, 36], [11, 34], [12, 26], [13, 25], [14, 24], [14, 57], [15, 23], [16, 17], [16, 22], [17, 18], [17, 19], [17, 21], [18, 19], [18, 20], [19, 20], [20, 21], [22, 23], [40, 41], [41, 42], [42, 43], [46, 69], [47, 68], [49, 50], [49, 51], [51, 64], [52, 63], [53, 62], [54, 61], [55, 58], [55, 59], [55, 60], [58, 60]]
    pairs [[6, 92], [7, 91], [8, 90], [9, 89], [10, 88], [11, 54], [12, 53], [13, 52], [14, 51], [16, 38], [17, 31], [18, 30], [19, 29], [21, 28], [22, 27], [23, 26], [60, 85], [61, 84], [62, 83], [66, 79], [67, 78], [68, 77], [69, 76]]
    123456789112345678921234567893123456789412345678951234567896123456789712345678981234567899123456789
    GGACUCGGGGUGCCCUUCUUGAAGGCUGAGAAAUACCCGUAUCACCUGAUCUGGAUAAUGCCAGCGUAGGGAAGUUC
    (((((((((.((((.(((..))))))......)..)))).....(((...((((......))))...)))..)))))
              x                      x                                                [11, 34]
                 x                                          x                         [14, 57]
                  x       x                                                           [15, 23]
                   x     x                                                            [16, 22]
    GGACUCGGGGUGCCCUUCUgcgUGAAGGCUGAGAAAUACCCGUAUCACCUGAUCUGGAUAAUGCCAGCGUAGGGAAGUUC
    (((((((((.((((.(((.....))))))......)..)))).....(((...((((......))))...)))..)))))
    FINAL MAPPING //////////////////////////////////////////////////////////////////////////////////
    GGACUCGGGGUGCCCUUCUgcgUGAAGGCUGAGAAAUACCCGUAUCACCUGAUCUGGAUAAUGCCAGCGUAGGGAAGUUC
    (((((((((.((((.(((.....))))))......)..)))).....(((...((((......))))...)))..)))))
              x                         x                                             [11, 37]
                 x                                             x                      [14, 60]
                  x          x                                                        [15, 26]
                   x        x                                                         [16, 25]
                    x     x                                                           [17, 23]
    [[21, 47], [24, 70], [25, 36], [26, 35], [27, 33]]
    draw_dists([[21, 47], [24, 70], [25, 36], [26, 35], [27, 33]])
    output file: tpp.ec.txt_52_sselected.csv_mapped.csv

"""

import pandas as pd
import sys
import numpy as np
import argparse

from rna_tools.SecondaryStructure import parse_vienna_to_pairs

def rna_dca_mapping(seqfn, gseqfn, file_interactions, noss, noshort, offset, mss, verbose):
    """This function is deviede into

    .. warning:: in the line that we load parameters, watch for sep argument that defines seperator of your file (line21)
    """
    v = verbose

    #
    # Load the data, open files and parse information
    #
    # final ss and seq, ungapped
    f = open(seqfn)
    header = f.readline().strip()  # get rid of header
    seq = f.readline().strip()
    ss = f.readline().strip()
    # gapped
    f = open(gseqfn)
    header = f.readline().strip()  # get rid of header
    gseq = f.readline().strip()
    gss = f.readline().strip()
    # DCA
    df = pd.read_csv(file_interactions,sep=" ")
    interactions = zip(df['i'].tolist(), df['j'].tolist())
    #
    # Show input
    #
    # [(38, 51), (7, 110), (37, 52) from the input
    interactions.sort()
    print 'interactions:\n', interactions
    #
    # Process unmapped scores on gaped sequence
    # I panel
    #
    pairs = parse_vienna_to_pairs(gss)[0]
    print 'pairs', pairs
    print 'UNMAPPED SCORES ' + '/' * len(gseq)
    print '123456789112345678921234567893123456789412345678951234567896123456789712345678981234567899123456789'
    print gseq
    print gss

    for i in interactions:
        # form 0 or from 1 ?! ## be careful here! scores starts from 0 or 1 !?
        ij = [0,0]
        ij[0] = i[0] + 1
        ij[1] = i[1] + 1
        i = ij

        if noshort:
            delta = i[1] - i[0]
            if delta < 6:
                continue

        # remove interactions from gaps
        if gseq[i[0] - 1] == '-' or gseq[i[1] - 1] == '-':
            continue

        if noss:
            if i in pairs:
                continue

        line_new = 'x'.rjust(i[0]) + 'x'.rjust(i[1] - i[0]) + str(i).rjust(len(gseq) - i[1] + 10)
        print line_new
    #
    # How this mapping works, kurwa?
    # II panel
    #
    print 'MAPPED SCORES //' + '/' * len(gseq)
    mapped_interactions = []
    for i in interactions:
        ij = i
        #ij = [0,0]
        #ij[0] = i[0] - 1
        #ij[1] = i[1] - 1
        if v: print 'ij:', ij
        if v: print gseq[ij[0]], gseq[ij[1]]

        # ok, here I test if this pair comes from gaps, . -> -
        if gseq[ij[0]] == '-' or gseq[ij[1]] == '-':
            if v: print 'Removed Interaction:', ij
        else:
            [a,b]=[ij[0] - gseq[:ij[0]].count('-') +1, ij[1] - gseq[:ij[1]].count('-') + 1,]
            if v: print i, '->', a, b
            mapped_interactions.append([a, b])

    print 'Mapped Interactions:\n', mapped_interactions
    if v:
        for i in mapped_interactions:
            print str(i)

    pairs = parse_vienna_to_pairs(gss)[0]
    print 'pairs', pairs
    print '123456789112345678921234567893123456789412345678951234567896123456789712345678981234567899123456789'
    print gseq.replace('-','')  # what is the gap character, - or . ?
    print gss.replace('-', '')
    mapped_interactions.sort()

    filtered_interactions = []
    pairs = parse_vienna_to_pairs(gss.replace('-', ''))[0]
    for i in mapped_interactions:
        if noshort:
            delta = i[1] - i[0]
            if delta < 6:
                continue

        # remove if there is a gap
        if seq[i[0] - 1] == '-' or seq[i[1] - 1] == '-':
            continue

        if noss:
            if i in pairs:
                continue

        line_new = 'x'.rjust(i[0]) + 'x'.rjust(i[1] - i[0]) + str(i).rjust(len(seq) - i[1] + 10)
        print line_new
    #
    # How to include a gap in the mapping?
    # III panel, the final
    #
    print seq
    print ss

    pairs = parse_vienna_to_pairs(ss.replace('-', ''))[0]

    def n_lower_chars(string):
        """ https://stackoverflow.com/questions/10953189/count-lower-case-characters-in-a-string """
        return sum(1 for c in string if c.islower())

    print 'FINAL MAPPING //' + '/' * len(seq)
    nmapped_interactions = []
    for i in mapped_interactions:
        ij = i
        if v: print 'ij:', ij
        if v: print seq[ij[0] - 1], seq[ij[1] - 1]
        # gap mapping
        a = ij[0] + n_lower_chars(seq[:ij[0]])
        b = ij[1] + n_lower_chars(seq[:ij[1]])
        if v: print i, '->', a,b
        nmapped_interactions.append([a,b])
    mapped_interactions = nmapped_interactions

    print(seq)
    print(ss)
    pairs = parse_vienna_to_pairs(ss)[0]
    nmapped_interactions = []
    for i in mapped_interactions:
        if noshort:
            delta = i[1] - i[0]
            if delta < 6:
                continue

        if seq[i[0] - 1] == '-' or seq[i[1] - 1] == '-':
            continue

        if noss:
            if i in pairs:
                continue

        line_new = 'x'.rjust(i[0]) + 'x'.rjust(i[1] - i[0]) + str(i).rjust(len(seq) - i[1] + 10)
        nmapped_interactions.append([i[0], i[1]])
        print line_new
        if mss:
            print(ss)
    mapped_interactions = nmapped_interactions

    if offset:
        # e.g. filter_interaction is [[18, 71]], if offset is 10 then it will give you [[28, 81]]
        nmapped_interactions = [[x[0] + offset, x[1] + offset] for x in mapped_interactions]
    mapped_interactions = nmapped_interactions

    print mapped_interactions
    print 'draw_dists(' + str(mapped_interactions) + ')'
    print 'output file:', file_interactions+"_mapped.csv"
    a = pd.DataFrame(list(mapped_interactions), columns=["i","j"])
    a.to_csv(file_interactions+"_mapped.csv",sep=" ")


def get_parser():
    parser = argparse.ArgumentParser()  # description=__doc__ , formatter_class=argparse.RawDescriptionHelpFormatter)
    parser.add_argument('--seq', help='seq fn in Fasta format', required=True)
    parser.add_argument('--gseq', help='gapped sequence and secondary structure (like in the alignment used for DCA) in Fasta format', required=True)
    parser.add_argument('--dca', help='file with parsed interactions', required=True)
    parser.add_argument('--offset', help="offset", type=int)
    parser.add_argument('--noss', help='filter out ss from plot', action='store_true')
    parser.add_argument('--mss', help='ss every each line', action='store_true')
    parser.add_argument('--verbose', help='be verbose', action='store_true')
    parser.add_argument('--noshort', help='filter out short interactions, dist in seq < 6 nt', action='store_true')
    return parser

if __name__=="__main__":
    parser = get_parser()
    args = parser.parse_args()
    rna_dca_mapping(args.seq, args.gseq, args.dca, args.noss, args.noshort, args.offset, args.mss, args.verbose)



VARNA_PATH = '/Users/magnus/work/opt/varna'
VARNA_JAR_NAME = 'VARNA.jar'
SIMRNA_DATA_PATH = '/Users/magnus/work/opt/simRNA/SimRNA_64bitIntel_MacOSX_staticLibs/data'
QRNAS_PATH = "/Users/magnus/work/opt/qrnas/"
RCHIE_PATH = "/Users/magnus/work/opt/r-chie/"
RFAM_DB_PATH = "/Users/magnus/work/db/rfam/Rfam.cm"
CONTEXTFOLD_PATH = "/Users/magnus/work/opt/ContextFold_1_00/"
DIFF_TOOL = "open -a diffmerge"
CPUS_CLUSTER = 630
#RNASTRUCTURE_PATH = "/Users/magnus/work/opt/RNAstructure/6.0.1/"
RNASTRUCTURE_PATH = "/Users/magnus/work/opt/RNAstructure/6.1/"
ENTRNA_PATH = "/Users/magnus/work/opt/ENTRNA"



#!/usr/bin/env python

"""Input as a file::

  >ade
  GCU-U-CAUAUAAUCCUAAUGAUAUGG-UUUGGGA-GUUUCUACCAAGAG-CC--UUAAA-CUCUU---GAUUAUG-AAGU-
  (((.(.((((,,,(((((((_______.))))))).,,,,,,,,(((((((__.._____))))))...),,)))).)))).

to get::

  >ade
  GCUUCAUAUAAUCCUAAUGAUAUGGUUUGGGAGUUUCUACCAAGAGCCUUAAACUCUUGAUUAUGAAGU
  ((((((((...(((((((.......)))))))........((((((.......))))))..))))))))

"""
from rna_tools.tools.rna_alignment.rna_alignment import clean_seq_and_ss
import argparse

def get_parser():
    parser = argparse.ArgumentParser(description=__doc__, formatter_class=argparse.RawDescriptionHelpFormatter)
    parser.add_argument('file', help="subsection of an alignment")
    return parser

if __name__ == '__main__':
    args = get_parser().parse_args()
    f = open(args.file)
    header = f.readline().strip()
    seq = f.readline()
    ss = f.readline()
    nseq, nss = clean_seq_and_ss(seq, ss)
    print(header)
    print(nseq)
    print(nss)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""rna_csv2pairs.py - get pairs based on a csv file.

Usage::

    $ rna_csv2pairs.py rp06_MohPairs.txt
    [[5, 42], [11, 26], [24, 89], [26, 44], [26, 75], [35, 75],

Input::

    $ head rp06_MohPairs.txt
    11,26
    26,44
    5,42

"""
from __future__ import print_function

import argparse


def get_parser():
    parser = argparse.ArgumentParser(
        description=__doc__, formatter_class=argparse.RawDescriptionHelpFormatter)

    parser.add_argument('fn', help="csv file", default="")
    return parser


if __name__ == '__main__':
    parser = get_parser()
    args = parser.parse_args()

    pairs = []
    for l in open(args.fn):
        if not l.startswith('#'):
            a, b = l.strip().split(',')
            a = int(a)
            b = int(b)
            pairs.append([a, b])
    pairs.sort()
    print(pairs)



from rna_tools.tools.clarna_app import clarna_app

if __name__ == '__main__':
    ss = '((((.[[[[[[.))))........((((.....]]]]]]...(((((....)))))..))))'
    fnCRref = clarna_app.get_ClaRNA_output_from_dot_bracket(ss)
    f = '../rna_calc_rmsd/test_data/pistol/5k7c_clean_onechain_renumber_as_puzzle_srr.pdb'
    fnCR = clarna_app.clarna_run(f, force=False)
    results = clarna_app.clarna_compare(fnCRref, fnCR)
    print(results) #
    #tmp_Z42i_..pdb.outCR     5k7c_clean_onechain_renumber_as_puzzle_srr.pdb.outCR      0.706      NA         0.865      NA         0.842      0.889      NA         0.000



from pymol import cmd,util
import inspect
import tempfile
from glob import glob
from tempfile import NamedTemporaryFile
# Pymol commands used by the Das Lab
# (C) R. Das 2010-2013.
#
# Some documentation and sample images available at:
#
# https://docs.google.com/document/d/1uWeEEGPjAceaw07ESf9bec-FrxW4Bx6jGaBqoHbSXuo/edit
#

print('rp06() -- colors for helices of rp06')
print('rp14() -- colors for helices of rp14')
print('sa -- superimpose all')
print('aa -- align all -- full atom RMSD!')
print('rcd() # flat ribbon with sticks')

def sa(intra=False,rainbow=True):
  """
  Superimpose all open models onto the first one.
  This may not work well with selections.
  Option intra can be set to True to enable intra_fit first, for working with multi-state (nmr) pdbs.
  [Thanks to Kyle Beauchamp for this one]
  """
  AllObj=cmd.get_names("all")
  for x in AllObj:
    print(AllObj[0],x)
    if intra==True:
      cmd.intra_fit(x)
    if rainbow==True:
      cmd.util.chainbow(x)
    cmd.align(x,AllObj[0])
    cmd.zoom()

def superimpose_all(intra=False,rainbow=True):
  sa( intra, rainbow );

def chainbow():
  """
  run chainbow on all molecules, one by one.
  """
  AllObj=cmd.get_names("all")
  for x in AllObj:
    print(AllObj[0],x)
    cmd.util.chainbow(x)

def color_by_data( filename, offset = 0, min_val=-1.0, max_val = 0.0 ):
  """
  Read in a text file with rows like:

  125 0.12
  126 1.50

  and color specified residue numbers by scalar values.
  Takes advantage of B-factor column, and color by temperature
  function in pymol. Note that coloring is scaled/offset based
  on lowest/highest scalar value.
  """
  lines = open( filename ).readlines()
  data = {}
  data_backbone = {}

  avg_data = 0.0
  min_data = 0.0
  max_data = 0.0
  for line in lines:
    cols = string.split( line )
    dataval = float( cols[1] )
    if min_val >= 0 and dataval < min_val: dataval = min_val
    if max_val > 0 and dataval > max_val: dataval = max_val
    data[ int( cols[0] )  ] = dataval
    avg_data = avg_data + dataval
    if ( dataval < min_data ): min_data = dataval
    if ( dataval > max_data ): max_data = dataval

    if len( cols ) > 2:
      dataval2 = float( cols[2] )
      if min_val >= 0 and dataval2 < min_val: dataval2 = min_val
      if max_val > 0 and dataval2 > max_val: dataval2 = max_val
      data_backbone[ int( cols[0] ) ] = dataval2


  avg_data /= len( data.keys() )

  cmd.alter( 'all', 'b=%6.3f' % avg_data )

  for i in data.keys():
    cmd.alter( 'resi  \\%d' % (i+int(offset)),  'b=%6.3f' % data[i] )

  backbone_tag = " and (name o1p+o2p+o3p+p+op1+op2+'c1*'+'c2*'+'c3*'+'c5*'+'o2*'+'o3*'+'o4*'+'o5*'+'c1*'+'c2*'+'c3*'+'c4*'+'o2*'+'o4*'+c1'+c2'+c3'+c5'+o2'+o3'+o4'+o5'+c1'+c2'+c3'+c4'+o2'+o4') and (not name c1+c2+c3+c4+c5+o2+o3+o4+o5)"
  for i in data_backbone.keys():
    cmd.alter( 'resi  \\%d %s' % (i+int(offset),backbone_tag),  'b=%6.3f' % data_backbone[i] )

  if ( min_val < 0 ): min_val = min_data
  if ( max_val < 0 ): max_val = max_data

  cmd.spectrum( "b", "rainbow","all",min_val,max_val )
  #cmd.ramp_new("ramp_obj", "1gid_RNAA", range=[0, 0, max_val], color="[blue, white, red ]")

def align_all( subset = [] ):
  """
  Superimpose all open models onto the first one.
  This may not work well with selections.
  """
  PATH_BAR_CHAT = '/Users/magnus/Documents/GTD/software/data_hacks/bar_chart.py'

  print("""This returns a list with 7 items:

    RMSD after refinement
    Number of aligned atoms after refinement
    Number of refinement cycles
    RMSD before refinement
    Number of aligned atoms before refinement
    Raw alignment score
    Number of residues aligned """)

  AllObj=cmd.get_names("all")
  report = []
  for x in AllObj[1:]:
    #print(AllObj[0],x)
    subset_tag = ''
    if isinstance( subset, int ):
      subset_tag = ' and resi %d' % subset
    elif isinstance( subset, list ) and len( subset ) > 0:
      subset_tag = ' and resi %d' % (subset[0])
      for m in range( 1,len(subset)): subset_tag += '+%d' % subset[m]
    elif isinstance( subset, str ) and len( subset ) > 0:
      subset_tag = ' and %s' % subset
    values = cmd.align(x+subset_tag,AllObj[0]+subset_tag)
    print(AllObj[0], x, ' '.join([str(v) for v in values]), '-- RMSD', values[3], ' of ', values[6], 'residues')
    print(AllObj[0], x, 'RMSD: ', values[3], ' of ', values[6], 'residues')
    report.append([AllObj[0], x, values[3], values[6]])
    cmd.zoom()


  print('==== SUMMARY =====')
  f = NamedTemporaryFile(delete=False)
  for i in report:
    if not i[1].startswith('_align'):
      # rp14_5ddp_bound_clean_ligand rp14_farna_eloop_nol2fixed_cst.out.1 RMSD:  4.49360132217 of 52 residues
      if i[1] not in ['sele', 'rov_pc']: # skip them
        print(i[0], i[1], 'RMSD:', round(i[2],2), str(i[3]) + 'nt')
        #f.write(i[0] + '-' + i[1] + ' ' + str(i[2]) + '\n')
  print()
  #print f.name
  #x = 'cat ' + f.name + ' | python ' + PATH_BAR_CHAT + ' -A -k -v --dot ='
  #f.close()
  #print x
  #print commands.getoutput(x)


def render_molecules():
  rd()

def rd():
  """
  rhiju's favorite coloring of proteins and generic molecules
  side chains are all-heavy-atom and colored CPK, backbone is
  rainbow cartoon from N to C terminus.
  """
  cmd.bg_color( "white" )
  AllObj=cmd.get_names("all")

  for x in AllObj:
    #print(AllObj[0],x)
    print(x)
    cmd.show( "cartoon", x )
    cmd.hide( "line", x )
    cmd.color( "white", x+" and elem C" )
    cmd.color( "blue", x+" and elem N" )
    cmd.color( "red", x+" and elem O" )
    cmd.color( "yellow", x+" and elem S" )
    cmd.spectrum( "count", "rainbow", x+" and name CA+C" )
    cmd.show( "sticks", x +" and not elem H and not name C+N+O" )
    cmd.show( "sticks", x +" and resn PRO and name N" )
    cmd.show( "sticks", x + " and name NR+CR+CS+CP+CQ" )
    cmd.show( "sticks", x + " and not elem H and neighbor name NR+CQ+CR+CS+CP" )
    cmd.show( "sticks", x + " and not elem H and neighbor neighbor name NR+CQ+CR+CS+CP" )
    cmd.set( "cartoon_oval_width", 0.1 )
    cmd.set( "cartoon_oval_length", 0.5 )

def rx():
  """
  rhiju's favorite coloring of proteins, more details --
  no cartoon; heavy backbone
  """
  cmd.bg_color( "white" )
  AllObj=cmd.get_names("all")

  for x in AllObj:
    #print(AllObj[0],x)
    print(x)
    cmd.hide( "line", x )
    cmd.color( "white", x+" and elem C" )
    cmd.color( "blue", x+" and elem N" )
    cmd.color( "red", x+" and elem O" )
    cmd.color( "yellow", x+" and elem S" )
    cmd.spectrum( "count", "rainbow", x+" and name CA+C" )
    #cmd.show( "sticks", x +" and not elem H and not name C+N+O" )

    cmd.select('backbone','name o+c+ca+n')
    cmd.show('sticks','not elem H')

    if not x.count( 'BACKBONE' ):
      cmd.create( x+"_BACKBONE", x+" and not element H and backbone" )


    cmd.set('stick_radius', '0.5', "*BACKBONE" )

def render_x():
  rx()

def rj():
  """
  rhiju's residue-level favorite coloring of proteins
  """
  cmd.bg_color( "white" )
  AllObj=cmd.get_names("all")

  for x in AllObj:
    #print(AllObj[0],x)
    print(x)
    cmd.show( "cartoon", x )
    #cmd.hide( "line", x )
    cmd.show( "line", x )
    cmd.color( "gray", x+" and resn trp+phe+ala+val+leu+ile+pro+met" )
    cmd.color( "orange", x+" and resn gly" )
    cmd.color( "red", x+" and resn asp+glu" )
    cmd.color( "blue", x+" and resn lys+arg+his" )
    cmd.color( "purple", x+" and resn cys" )
    cmd.color( "forest", x+" and resn tyr+thr+ser+gln+asn" )
    #cmd.spectrum( "count", "rainbow", x+" and name CA" )
    cmd.show( "sticks", x +" and not elem H and not name C+N+O" )
    cmd.show( "sticks", x +" and resn PRO and name N" )
    cmd.hide( "sticks", x + " and name NR+CR+CS+CP+CQ" )
    cmd.show( "sticks", x + " and not elem H and neighbor name NR+CQ+CR+CS+CP" )
  cmd.set( "cartoon_rect_length", 0.75 )
  cmd.set( "cartoon_rect_width", 0.1 )
  cmd.set( "cartoon_oval_length", 0.6 )
  cmd.set( "cartoon_oval_width", 0.2 )

def render_rhiju():
  rj()


def rr(rainbow=False):
  """
  rhiju's favorite coloring of RNA
  with 2' OH as spheres,
  bases as filled rings, and backbone as cartoon
  ribbons, rainbow colored from 5' to 3'. No hydrogens,
  white background.
  """
  cmd.bg_color( "white" )

  cmd.hide('everything' )
  cmd.show('sticks','not elem H')

  #cmd.color( 'red','resn rG+G+DG')
  #cmd.color( 'forest','resn rC+C+DC')
  #cmd.color( 'orange','resn rA+A+DA')
  #cmd.color( 'blue','resn rU+U+DT+BRU')

  #cmd.set( 'cartoon_ring_color',  'red','resn rG+G+DG')
  #cmd.set( 'cartoon_ring_color',  'forest','resn rC+C+DC')
  #cmd.set( 'cartoon_ring_color',  'orange','resn rA+A+DA')
  #cmd.set( 'cartoon_ring_color',  'blue','resn rU+U+DT+BRU')

  #cmd.select('bases','name c2+c4+c5+c6+c8+n1+n2+n3+n4+n6+n7+n9+o2+o4+o6+n1p')
  #cmd.select('backbone', 'name o1p+o2p+o3p+p+c1*+c2*+c3*+c5*+o2*+o3*+o4*+o5*')
  #cmd.select('sugar', 'name c1*+c2*+c3*+c4*+o2*+o4*')
  AllObj=cmd.get_names("all")

  cmd.color( 'red','resn rG+G and name N1+C6+O6+C5+C4+N7+C8+N9+N3+C2+N1+N2')
  cmd.color( 'forest','resn rC+C and name N1+C2+O2+N3+C4+N4+C5+C6')
  cmd.color( 'orange','resn rA+A and name N1+C6+N6+C5+N7+C8+N9+C4+N3+C2')
  cmd.color( 'blue','resn rU+U and name N3+C4+O4+C5+C6+N1+C2+O2')

  cmd.select( 'backbone', " (name o1p+o2p+o3p+p+op1+op2+'c1*'+'c2*'+'c3*'+'c5*'+'o2*'+'o3*'+'o4*'+'o5*'+'c1*'+'c2*'+'c3*'+'c4*'+'o2*'+'o4*'+c1'+c2'+c3'+c5'+o2'+o3'+o4'+o5'+c1'+c2'+c3'+c4'+o2'+o4') and (not name c1+c2+c3+c4+c5+o2+o3+o4+o5) ")

  for x in AllObj:
    cmd.show( "cartoon", x )
    #cmd.spectrum( "count", "rainbow", x+" and backbone" )
    #cmd.color( 'white', 'backbone' )

  cmd.cartoon("tube", "backbone" )

  cmd.set( "cartoon_ring_mode", 3 )
  cmd.set( "cartoon_ring_transparency", 0.0 )
  cmd.set( "cartoon_tube_radius", 0.8)

  cmd.hide( "sticks", "backbone" )

  cmd.delete('backbone')


  cmd.alter( "name o2*","vdw=0.5" )
  cmd.show( "spheres", "name o2'+'o2*' and not name o2" )
  cmd.show( "sticks", "name 'o2*'+'c2*'" )

  cmd.alter( "resn mg", "vdw=1.0")
  cmd.alter( "resn hoh", "vdw=0.5")
  cmd.show( "spheres", "resn mg+sr+co+zn")

def render_rna():
  rr()

def rrs():
  """)
  rhiju's favorite coloring of RNA, showing
  all heavy atoms as sticks -- more detail than rr().
  """
  rr()
  cmd.show( 'sticks', 'not elem H' )

def render_rna_sticks():
  rr()

def rr2():
  """
  rhiju's favorite coloring of RNA, showing
  all heavy atoms as sticks -- more detail than rr().
  """
  rr()
  cmd.hide( 'spheres' )
  cmd.hide( 'sticks' )
  cmd.set( "cartoon_ring_mode", 0 )

def render_rna2():
  rr2()


def get_residue_colors( sele ):
  """
  Get RGB color values associated with a selection.
  Useful if you want to exactly match coloring of 3D models
  with coloring in, say, a MATLAB script.
  """
  pymol.stored.colors = []
  cmd.iterate( sele, "stored.colors.append( (chain, resi, name, color))")
  res_colors = {}
  for chain, resi, name, color in pymol.stored.colors:
    if name == 'CA': # c-alpha atom
      res_colors[(chain, resi)] = cmd.get_color_tuple(color)
  print(res_colors)
  return res_colors

def spr():
  """
  Load up these commands again after, say, an edit.
  """

  cmd.do( 'run '+inspect.getfile(inspect.currentframe()) )

def source_pymol_rhiju():
  """
  Load up these commands again after, say, an edit.
  """
  spr()


def loop_color( start, end, native=None, zoom=False ):
  """
  Used for rendering protein loop modeling puzzles.
  White in background, colored red/blue over loop.
  """

  rd()

  cmd.color( "white", "not resi %d-%d" % (start,end) )
  #cmd.hide( "cartoon", "resi %d-%d" % (start,end) )
  #cmd.show( "sticks", "not elem H and resi %d-%d" % (start,end) )

  #before_start = start - 1
  #cmd.show( "sticks", "name C and resi %d" % (before_start) )
  #after_end = end + 1
  #cmd.show( "sticks", "name N and resi %d" % (after_end) )

  cmd.color( "salmon",  "elem C and resi %d-%d" % (start,end) )

  #cmd.show( "lines", "not elem H" )
  #cmd.hide( "cartoon",  "resi %d-%d" % (start,end) )
  #cmd.show( "sticks",  "name C+N+CA+O and resi %d-%d" % (start,end) )
  cmd.hide( "sticks", "resi %d-%d and name C+N+O" % (start,end) )
  cmd.show( "sticks", "resn PRO and name N")
  cmd.show( "sticks", x +" and ( not elem H and neighbor name NR+CR+CS+CP+CQ )" )


  if native:

    # reassign colors based on native -- spectrum colors by atom count and
    # messes up loop coloring on small loop subsegments.
    #colors = get_residue_colors( "%s and resi %d-%d" % (native,start,end) )
    #for x in AllObj:
      #for m in range( start, end+1):
        #cmd.set_color( 'color%d' % m, colors[ ('','%d' % m) ] )
        #cmd.color( 'color%d' % m, 'elem C and resi %d' % m )


    cmd.color( "white", native + " and not resi %d-%d" % (start,end) )
    #cmd.color( "palecyan", native+" and not name C+N+CA+O")
    cmd.color( "skyblue", native+" and elem C and resi %d-%d" % (start,end) )

  if zoom: cmd.zoom( "resi %d-%d" % (start,end) )


def rb():
  """
  basic cartoon coloring
  """

  AllObj=cmd.get_names("all")
  cmd.bg_color( "white" )
  cmd.hide( "ev" )
  cmd.show( "cartoon" )
  cmd.cartoon( "rectangle" )
  cmd.set( "cartoon_ring_mode", 1 )
  cmd.set( "cartoon_rect_length", 0.7 )
  cmd.set( "cartoon_rect_width", 0.2 )
  for x in AllObj:
    print(AllObj[0],x)
    cmd.spectrum( "count", "rainbow", x )

def atomcolor():
  """
  atom coloring
  """

  cmd.bg_color( "white" )
  cmd.hide( "ev" )
  cmd.show( "sticks", "not elem H" )
  cmd.show( "lines", "elem H" )
  util.cbag()
  cmd.color( "white", "elem C" )

def rc():
  """
  tube coloring for large RNA comparisons
  """
  cmd.bg_color( "white" )
  cmd.hide( 'everything' )

  cmd.color( 'red','resn rG+G+DG')
  cmd.color( 'forest','resn rC+C+DC')
  cmd.color( 'orange','resn rA+A+DA')
  cmd.color( 'blue','resn rU+U+DT+BRU')

  AllObj=cmd.get_names("all")

  cmd.select( 'backbone', " (name o1p+o2p+o3p+p+op1+op2+'c1*'+'c2*'+'c3*'+'c5*'+'o2*'+'o3*'+'o4*'+'o5*'+'c1*'+'c2*'+'c3*'+'c4*'+'o2*'+'o4*'+c1'+c2'+c3'+c5'+o2'+o3'+o4'+o5'+c1'+c2'+c3'+c4'+o2'+o4') and (not name c1+c2+c3+c4+c5+o2+o3+o4+o5) ")

  for x in AllObj:
    print(x)
    cmd.show( "cartoon", x )
    cmd.spectrum( "count", "rainbow", x+" and backbone" )

  cmd.cartoon( "tube", "backbone" )

  cmd.set( "cartoon_ring_mode", 0 )
  cmd.set( "cartoon_ring_transparency", 0.0 )
  cmd.set( "cartoon_tube_radius", 1.0 )

  cmd.color( 'red','resn rG+G and name n1+c6+o6+c5+c4+n7+c8+n9+n3+c2+n1+n2')
  cmd.color( 'forest','resn rC+C and name n1+c2+o2+n3+c4+n4+c5+c6')
  cmd.color( 'orange','resn rA+A and name n1+c6+n6+c5+n7+c8+n9+c4+n3+c2')
  cmd.color( 'blue','resn rU+U and name n3+c4+o4+c5+c6+n1+c2+o2')

  cmd.delete('backbone')

def rcm():
  """
  tube coloring for large RNA comparisons
  """
  #cmd.bg_color( "white" )
  cmd.hide( 'everything' )

  cmd.set('cartoon_ladder_mode', 0 )

  cmd.color( 'red','resn rG+G+DG')
  cmd.color( 'forest','resn rC+C+DC')
  cmd.color( 'orange','resn rA+A+DA')
  cmd.color( 'blue','resn rU+U+DT+BRU')

  AllObj=cmd.get_names("all")

  cmd.select( 'backbone', " (name o1p+o2p+o3p+p+op1+op2+'c1*'+'c2*'+'c3*'+'c5*'+'o2*'+'o3*'+'o4*'+'o5*'+'c1*'+'c2*'+'c3*'+'c4*'+'o2*'+'o4*'+c1'+c2'+c3'+c5'+o2'+o3'+o4'+o5'+c1'+c2'+c3'+c4'+o2'+o4') and (not name c1+c2+c3+c4+c5+o2+o3+o4+o5) ")

  for x in AllObj:
    print(x)
    cmd.show( "cartoon", x )
    cmd.spectrum( "count", "rainbow", x+" and backbone" )

  cmd.cartoon( "tube", "backbone" )

  cmd.set( "cartoon_ring_mode", 0 )
  cmd.set( "cartoon_ring_transparency", 0.0 )
  cmd.set( "cartoon_tube_radius", 1.0 )

  cmd.color( 'red','resn rG+G and name n1+c6+o6+c5+c4+n7+c8+n9+n3+c2+n1+n2')
  cmd.color( 'forest','resn rC+C and name n1+c2+o2+n3+c4+n4+c5+c6')
  cmd.color( 'orange','resn rA+A and name n1+c6+n6+c5+n7+c8+n9+c4+n3+c2')
  cmd.color( 'blue','resn rU+U and name n3+c4+o4+c5+c6+n1+c2+o2')

  cmd.delete('backbone')


def rcd():
  """
  fancy ribbon coloring for large RNA comparisons
  """
  rc()
  cmd.cartoon( 'dumbbell')
  cmd.set( 'cartoon_dumbbell_radius', 0.5 )

def render_cartoon():
  rc()

def bx():
  txt = """color black, all
color pink, resi 3-5+132-134 # p1
color forest, resi 12-13+129-130 # p3
color orange, resi 17-33 # p4
color green, resi 36-45  # p5
color blue, resi 47-124 # p6color
  """
  for t in txt.split('\n'):
    color, resi = t.replace('color ', '').split(',')
    print(color, resi)
    cmd.color(color.strip(), resi.strip())

def color_by_text(txt):
  for t in txt.strip().split('\n'):
    color, resi = t.replace('color ', '').split(',')
    print(color, resi)
    cmd.color(color.strip(), resi.strip())




from multiprocessing import Process

cmd.extend('rib', rcm)
#cmd.extend('clr_', clr)
cmd.extend('aa', align_all)
cmd.extend('rr', rr)
cmd.extend('rcd', rcd)



#!/usr/bin/env python
from __future__ import print_function
__docformat__ = 'reStructuredText'
import os
import Bio.PDB.PDBParser
import Bio.PDB.Superimposer
from Bio.PDB.PDBIO import Select
from Bio.PDB import PDBIO
from Bio.SVDSuperimposer import SVDSuperimposer
from numpy import sqrt, array, asarray

class RNAmodel:
    """RNAmodel

    :Example:

        >>> rna = RNAmodel("test_data/rp14/rp14_5ddp_bound_clean_ligand.pdb", [1], False, None)
        >>> rna.get_report()
        "File:  rp14_5ddp_bound_clean_ligand.pdb  # of atoms: 1 \\nresi:  1  atom:  <Atom C3'> \\n"

    :param fpath: file path, string
    :param residues: list of residues to use (and since we take only 1 atom, C3', this equals to number of atoms.
    :param save: boolean, save to output_dir or not
    :param output_dir: string, if save, save segments to this folder
    """
    #:returns: None
    #:rtype: None
    #"""
    def __init__(self, fpath, residues, save=False, output_dir=""):

        # parser 1-5 -> 1 2 3 4 5
        self.struc = Bio.PDB.PDBParser().get_structure('', fpath)
        self.fpath = fpath
        self.fn = os.path.basename(fpath)
        self.residues = residues #self.__parser_residues(residues)
        self.__get_atoms()
        #self.atoms = []
        if save:
            self.save(output_dir) # @save

    def __parser_residues(self, residues):
        """Get string and parse it
        '1 4 5 10-15' -> [1, 4, 5, 10, 11, 12, 13, 14, 15]"""
        rs = []
        for r in residues.split():
            l = parse_num_list(r)
            for i in l:
                if i in rs:
                    raise Exception('You have this resi already in your list! See', residues)
            rs.extend(l)
        return rs

    def __get_atoms(self):
        self.atoms = []
        for res in self.struc.get_residues():
            if res.id[1] in self.residues:
                self.atoms.append(res["C3'"])
                #print res.id
                #ref_atoms.extend(, ref_res['P'])
            #ref_atoms.append(ref_res.get_list())
        if len(self.atoms) <= 0:
            raise Exception('problem: none atoms were selected!: %s' % self.fn)
        return self.atoms

    def __str__(self):
        return self.fn #+ ' # beads' + str(len(self.residues))

    def __repr__(self):
        return self.fn #+ ' # beads' + str(len(self.residues))

    def get_report(self):
        """Str a short report about rna model"""
        t = ' '.join(['File: ', self.fn, ' # of atoms:', str(len(self.atoms)), '\n'])
        for r, a in zip(self.residues, self.atoms):
            t += ' '.join(['resi: ', str(r), ' atom: ', str(a), '\n'])
        return t

    def get_rmsd_to(self, other_rnamodel, output='', dont_move=False):
        """Calc rmsd P-atom based rmsd to other rna model"""
        sup = Bio.PDB.Superimposer()

        if dont_move:
            # fix http://biopython.org/DIST/docs/api/Bio.PDB.Vector%27.Vector-class.html
            coords = array([a.get_vector().get_array() for a in self.atoms])
            other_coords = array([a.get_vector().get_array() for a in other_rnamodel.atoms])
            s = SVDSuperimposer()
            s.set(coords,other_coords)
            return s.get_init_rms()

        try:
            sup.set_atoms(self.atoms, other_rnamodel.atoms)
        except:
            print(self.fn, len(self.atoms),  other_rnamodel.fn, len(other_rnamodel.atoms))
            for a,b in zip(self.atoms, other_rnamodel.atoms):
                print(a.parent, b.parent)#a.get_full_id(), b.get_full_id())

        rms = round(sup.rms, 3)
        
        if output:
            io = Bio.PDB.PDBIO()
            sup.apply(self.struc.get_atoms())
            io.set_structure( self.struc )
            io.save("aligned.pdb")

            io = Bio.PDB.PDBIO()
            sup.apply(other_rnamodel.struc.get_atoms())
            io.set_structure( other_rnamodel.struc )
            io.save("aligned2.pdb")
        return rms

    def save(self, output_dir, verbose=True):
        """Save structures and motifs """
        folder_to_save =  output_dir + os.sep # ugly hack 'rp14/'
        try:
            os.makedirs(folder_to_save)
        except OSError:
            pass

        try:
            os.mkdir(folder_to_save + 'structures')
        except OSError:
            pass

        try:
            os.mkdir(folder_to_save + 'motifs')
        except OSError:
            pass

        RESI = self.residues
        if not self.struc:
            raise Exception('self.struct was not defined! Can not save a pdb!')

        class BpSelect(Select):
            def accept_residue(self, residue):
                if residue.get_id()[1] in RESI:
                    return 1
                else:
                    return 0

        io = PDBIO()
        io.set_structure(self.struc)
        fn = folder_to_save + 'structures' + os.sep + self.fn #+ '.pdb'
        io.save(fn)
        if verbose:
            print('    saved to struc: %s ' % fn)

        io = PDBIO()
        io.set_structure(self.struc)
        fn = folder_to_save +  'motifs/' + os.sep + self.fn #+ self.fn.replace('.pdb', '_motif.pdb')# #+ '.pdb'
        io.save(fn, BpSelect())
        if verbose:
            print('    saved to motifs: %s ' % fn)

#main
if __name__ == '__main__':
    import doctest
    doctest.testmod()

    #rna = RNAmodel("test_data/rp14/rp14_5ddp_bound_clean_ligand.pdb", [1], False, Non3e)
    #print(rna.get_report())
    a = RNAmodel("test_data/GGC.pdb", [46,47,48])
    b = RNAmodel("test_data/GUC.pdb", [31, 32, 33])

    print(a.get_rmsd_to(b))
    print(a.get_rmsd_to(b, dont_move=True))
    



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# flake8: ignore=E501
r"""rna_pdb_edit_occupancy_bfactor.py - edit occupancy or bfactor in PDB file.

Example::

   rna_pdb_edit_occupancy_bfactor.py --occupancy --select A:1-40,B:1-22 \
                                    --set-to 0 \
                                    19_Bujnicki_Human_4_rpr_n0-000001.pdb


   rna_pdb_edit_occupancy_bfactor.py --occupancy \
                                     --select A:1-2 \
                                     --select-atoms P+C4\' \
                                     --set-to 10 \
                                     -o test_data/3w3s_homologymodel_out.PD
                                     --set-not-selected-to 8
                                     test_data/3w3s_homologymodel.pdb

"""
from __future__ import print_function
import re
import string
import argparse

from Bio import PDB
from Bio.PDB import PDBIO
from Bio.PDB.Atom import PDBConstructionWarning

import sys
import warnings
warnings.simplefilter('ignore', PDBConstructionWarning)


def edit_occupancy_of_pdb(txt, pdb, pdb_out, bfactor, occupancy, set_to,
                          set_not_selected_to, select_atoms, v=False):
    """Change ouccupancy or bfactor of pdb file.

    Load the structure, and first set everything to be `set_not_selected_to`
    and then  set selected to `sel_to`.

    Args:

       txt (str): A:1-10, selection, what to change
       pdb (str): filename to read as an input
       pdb_out (str): filename to save an output
       bfactor (bool): if edit bfactor
       occupancy (bool): if edit occupancy
       set_to (float): set to this value, if within selection
       set_not_selected_to (float): set to this value, if not within selection
       select_atoms (str): P, P+C4\\', use + as a separator
       v (bool): be verbose

    Returns:

       bool: if OK, save an output to pdb_out

    .. warning:: this function requires BioPython
    """
    struc = PDB.PDBParser().get_structure('struc', pdb)

    # first loop to set everything to set_not_selected_to
    if set_not_selected_to:
        for s in struc:
            for c in s:
                for r in c:
                    for a in r:
                        if bfactor:
                            a.set_bfactor(set_not_selected_to)
                        if occupancy:
                            a.set_occupancy(set_not_selected_to)

    # parser selected
    txt = txt.replace(' ', '')
    l = re.split('[,:;]', txt)
    if v:
        print(txt)
        print(l)

    # second loop to set_to for selected
    for i in l:  # ['A', '1-10', '15', '25-30', 'B', '1-10']
        if i in string.ascii_letters:
            if v:
                print('chain', i)
            chain_curr = i
            continue

        if '-' in i:
            start, ends = i.split('-')
            if v:
                print(start, ends)

            if start > ends:
                print('Error: range start > end ' + i, file=sys.stderr)
                return False
            index = list(range(int(start), int(ends)+1))
        else:
            index = [int(i)]

        for i in index:
            if v:
                print(index)
            try:
                atoms = struc[0][chain_curr][i]
            except KeyError:
                if i == chain_curr:
                    print('Error: Chain ' + chain_curr +
                          ' not found in the PDB structure', file=sys.stderr)
                else:
                    print('Error: Residue ' + chain_curr + ':' + str(i) +
                          ' found in the PDB structure', file=sys.stderr)
                    return False

            for a in atoms:
                # if run it for all atoms
                if not select_atoms:
                    if bfactor:
                        a.set_bfactor(set_to)
                    if occupancy:
                        a.set_occupancy(set_to)
                else:  # if select_atoms
                    if '+' in select_atoms:
                            # only for striping atom names
                            select_atoms_list = [x.strip() for x in
                                                 select_atoms.split('+')]
                    else:
                            select_atoms_list = select_atoms
                    if a.id in select_atoms_list:
                        if bfactor:
                            a.set_bfactor(set_to)
                        if occupancy:
                            a.set_occupancy(set_to)

    io = PDBIO()
    io.set_structure(struc)
    io.save(pdb_out)
    print('Saved ', pdb_out)
    return True


def get_parser():
    """Get parser."""
    p = argparse.ArgumentParser(description=__doc__,
                                formatter_class=argparse.RawDescriptionHelpFormatter)

    group = p.add_mutually_exclusive_group(required=True)
    group.add_argument('--bfactor', action='store_true', help="set bfactor")
    group.add_argument('--occupancy', action='store_true', help="set occupancy")

    p.add_argument('--select', help='get chain, e.g A:1-10, works also for multiple chains'
                                    'e.g A:1-40,B:1-22')
    p.add_argument('--set-to', help='set value to, default is 1', default=1)
    p.add_argument('--set-not-selected-to', help='set value to, default is 0', default=0)
    p.add_argument('-o', '--output', help='file output')
    p.add_argument('--verbose', action='store_true', help="be verbose")
    p.add_argument('--select-atoms', help="select only given atoms"
                                          "can be only one atom, e.g. P"
                                          "or more, use \\' for prims, e.g. P+C4\\'")
    p.add_argument('file', help='file')
    return p


# main
if __name__ == '__main__':
    parser = get_parser()
    args = parser.parse_args()
    if not args.output:
        args.output = args.file.replace('.pdb', '_out.pdb')
    edit_occupancy_of_pdb(args.select, args.file, args.output, args.bfactor, args.occupancy,
                          float(args.set_to), float(args.set_not_selected_to),
                          args.select_atoms, args.verbose)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""

"""
from __future__ import print_function
from Seq import RNASequence
import argparse
import sys
import subprocess
import os


def get_parser():
    parser = argparse.ArgumentParser(
        description=__doc__, formatter_class=argparse.RawDescriptionHelpFormatter)

    parser.add_argument('--method', help="rnaeval", default="rnaeval")#, required=True)
    parser.add_argument("--seq", help="", default="")
    parser.add_argument("--file", help="", default="", nargs='+')
    parser.add_argument("--cstinfile", action="store_true", help="the 3rd file should have cst line, e.g. '((....))'")
    parser.add_argument("--cst", help="--cst '((....))'")
    parser.add_argument("-v", "--verbose",
                        action="store_true", help="be verbose")
    return parser


def exe(cmd):
    o = subprocess.Popen(
        cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    out = o.stdout.read().strip().decode()
    err = o.stderr.read().strip().decode()
    return out, err


def exe2(cmd):
    os.system(cmd)

def hr(t):
    print
    if t:
        print(t)
    print('-' * 80)
    
if __name__ == '__main__':
    parser = get_parser()
    args = parser.parse_args()
    cst = args.cst

    for file in args.file:

        hr(file)

        if args.seq:
            seq = args.seq

        if args.file:
            f = open(file)
            header = f.readline()
            seq_string = f.readline().strip()
            if args.cstinfile:
                cst = f.readline().strip()
                assert cst, 'Cst line empty, check the input file'

        try:
            print(seq_string)
        except NameError:
            parser.print_usage()
            parser.exit(1)

        if args.cst:
            cst = args.cst
        if cst:
            print(cst, '<= cst')

        if args.method == 'rnaeval' or args.method == 'all':
            seq = RNASequence(seq_string)
            energy = seq.eval(cst, verbose=args.verbose)
            print('rnaeval:', energy)

        if args.method == 'mcfold' or args.method == 'all':
            seq = RNASequence(seq_string)
            if cst:
                en, ss, comment = seq.predict_ss('mcfold', constraints=cst, verbose=args.verbose)
            else:
                en, ss, comment = seq.predict_ss('mcfold', verbose=args.verbose)
            #if not ss:
            #    ss = 'x'
            print('mcfold:', en, comment)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""rna_roseta_head.py - get a head of a Rosetta silent file.

Example::

     $ rna_rosetta_head.py -n 10000 thf.out
     # a new file will be created, thf_10000.out

Silent file::

    [peyote2] thf head -n 100 thf.out
    SEQUENCE: ggagaguagaugauucgcguuaagugugugugaaugggaugucgucacacaacgaagcgagagcgcggugaaucauugcauccgcucca
    SCORE:     score    rna_data_backbone    rna_vdw    rna_base_backbone    rna_backbone_backbone    rna_repulsive    rna_base_pair    rna_base_axis    rna_base_stagger    rna_base_stack    rna_base_stack_axis     rna_rg    atom_pair_constraint    linear_chainbreak       N_WC      N_NWC       N_BS description
    REMARK BINARY SILENTFILE
    SCORE:  -601.975                0.000     31.113              -16.960                   -3.888           20.501         -302.742          -38.531            -158.004           -80.764               -110.053     23.750                   0.000               33.601         32          6         86    S_000001_000
    FOLD_TREE  EDGE 1 4 -1  JEDGE 4 85 1  C4   C2   END  EDGE 4 5 -1  EDGE 85 80 -1  EDGE 85 89 -1  JEDGE 80 40 5  C4   C2   END  EDGE 80 78 -1  EDGE 40 43 -1  EDGE 40 33 -1  JEDGE 33 45 4  C4   C2   END  EDGE 45 54 -1  EDGE 45 44 -1  EDGE 33 20 -1  JEDGE 20 65 3  C2   C4   END  EDGE 65 67 -1  EDGE 20 17 -1  EDGE 65 63 -1  JEDGE 17 69 2  C4   C2   END  JEDGE 63 58 6  C4   C2   END  EDGE 17 6 -1  EDGE 58 59 -1  EDGE 63 60 -1  EDGE 69 77 -1  EDGE 58 55 -1  EDGE 69 68 -1  S_000001_000
    RT -0.987743 0.139354 0.0703103 0.135963 0.989404 -0.0509304 -0.0766626 -0.0407466 -0.996224 6.25631 0.103544 0.0647696   S_000001_000
    RT -0.98312 0.1587 -0.091045 0.166923 0.981743 -0.0912024 0.074909 -0.10486 -0.991662 5.89962 -1.95819 -0.1075   S_000001_000
    RT -0.987645 0.154078 0.0285994 0.153854 0.988044 -0.00989514 -0.0297821 -0.00537275 -0.999542 6.13138 1.047 0.115722   S_000001_000
    RT -0.989884 0.140722 0.0180554 0.140036 0.989532 -0.0348618 -0.0227723 -0.0319807 -0.999229 6.21787 0.201588 -0.0264223   S_000001_000
    RT -0.988455 0.134013 0.0706914 0.134924 0.990822 0.00825457 -0.0689364 0.0176973 -0.997464 6.19447 0.189237 0.125791   S_000001_000
    RT -0.990412 0.138036 0.00546261 0.137927 0.990299 -0.0168644 -0.00773751 -0.0159492 -0.999843 6.25456 0.0842849 -0.0135807   S_000001_000
    ANNOTATED_SEQUENCE: g[RGU:LowerRNA:Virtual_Phosphate]gaga[RAD:rna_cutpoint_lower]g[RGU:rna_cutpoint_upper]uagaugauucgcguuaagugugugugaaugggauguc[RCY:rna_cutpoint_lower]g[RGU:rna_cutpoint_upper]ucacacaacg[RGU:rna_cutpoint_lower]a[RAD:rna_cutpoint_upper]agcg[RGU:rna_cutpoint_lower]a[RAD:rna_cutpoint_upper]gagcgcg[RGU:rna_cutpoint_lower]g[RGU:rna_cutpoint_upper]ugaaucauu[URA:rna_cutpoint_lower]g[RGU:rna_cutpoint_upper]cauccgcucca[RAD:UpperRNA] S_000001_000
    Le3nY9smsa4zEMGdvAA+z+e

It seems to work::

    -rw-rw-r--   1 magnus users 474M 2017-08-06 05:25 thf_10000.out
    -rw -rw-r--   1 magnus users 947M 2017-08-06 04:54 thf.out

    [peyote2] thf rna_rosetta_n.py thf_10000.out
    10000

"""
from __future__ import print_function
import subprocess
import argparse
import sys


def get_parser():
    parser = argparse.ArgumentParser(
        description=__doc__, formatter_class=argparse.RawDescriptionHelpFormatter)
    parser.add_argument('-v', '--verbose', action='store_true')
    parser.add_argument('-n', '--nstruc', type=int, default=10000)
    parser.add_argument('file', help='ade.out')
    return parser


def run():
    """Pipline for modeling RNA."""
    args = get_parser().parse_args()
    nfile = ''
    c = -2
    for l in open(args.file):
        # print(l)
        if l.startswith('SCORE'):
            c += 1
            if c == int(args.nstruc):
                f = open(args.file.replace('.out', '_' + str(args.nstruc) + '.out'), 'w')
                # print(nfile)
                f.write(nfile)
                f.close()
                sys.exit(1)
        nfile += l


# main
if __name__ == '__main__':
    run()
